//     :     :     :     :     :     :     :     :     :     :
//   --------:  ---------:  ---------:  ---------:  ----     :
// ..|      |*..|       |*..|       |*..|       |*..|  |*....:.
//   |      |*  |       |*  |       |*  |       |*  |  |*    :
//   ---  ---*  |  |-----*  |  |-----*  |  |-----*  |  |*    :
// ....|  |*.:..|  |--****..|  |-----*..|  |--****..|  |*....:.
//     |  |* :  |    |*  :  |  |--  |*  |    |*  :  |  |*    :
//     |  |* :  |  |--*  :  |  |*|  |*  |  |--*  :  |  |*    :
// ....|  |*.:..|  |-----:..|  ---  |*..|  |-----:..|  -----.:.
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
// ....----*.:..---------*..---------*..---------*..--------*:.
//     :**** :   *********   *********   *********   ********:
//     :     :     :     :     :     :     :     :     :     :
//
// Version 0.81
// Copyright (c) 2002-2007
// Martijn "Amarillion" van Iersel.
//
// This file is part of the Tegel tilemapping add-on for allegro
//
//    Tegel is free software; you can redistribute it and/or modify
//    it under the terms of the GNU Lesser General 
//    Public License as published by the Free Software 
//    Foundation; either version 2.1 of the License, or
//    (at your option) any later version.
//
//    Tegel is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public License
//    along with Tegel; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
// official website:
// http://tegel.sourceforge.net/
//
// I hope you like this program. Please send 
// questions, comments, flames and feature 
// requests to amarillion@yahoo.com.
//
#include <MASkinG.h>
#include "mapbox.h"
#include "tilebox.h"
#include "layerbox.h"
#include "radarbox.h"
#include <string>
#include <deque>
#include "imagebutton.h"

#ifndef MAIN_DLG
#define MAIN_DLG

using namespace MAS;
using namespace std;

class mrud
{
public:
    string filename; // (of map)
    string datafile;
    string tilelist; // object name within datafile
};

typedef deque<mrud> mrudlist;

class MainDlg;
    
class MainDlgMapListener : public DataListener
{
    private:
        MainDlg *parent;
    public:
        MainDlgMapListener (MainDlg *p)
        {
            assert (p);
            parent = p;
        }
        virtual void changed (int code);        
};

class MainDlg : public Dialog {

	private:
		//widgets
		ClearScreen desktop;
		Shortcut pgup, pgdn, ctrl_n, ctrl_s, ctrl_c, ctrl_v, ctrl_l, ctrl_q, ctrl_z, ctrl_x, del, f1;
		MapWrapper mapwrap;
		MainDlgMapListener maplistener;
		LayerWrapper layerwrap;
		ScrollWrapper scrollwrap;
		MapBox mapbox;
		RadarBox radarbox;
		TileBox tilebox;
		LayerBox layerbox;
		Label lblFilename;
		Label lblCursor;
		Label lblLayerPalette;
	  
		enum {TB_NEW, TB_OPEN, TB_SAVE, TB_COPY, TB_PASTE, TB_BRUSH, TB_FILL, TB_SELECT};
		ImageButton toolbar[8];  

		Bitmap toolbarImage[8];
		Bitmap mouseCursor[4];
		
		char cwd[256]; // store current working directory
		char sscwd[256]; // cwd for mapshots
	  
		Menu menuFile, menuEdit, menuHelp, 
			menuTools, menuMain;
		enum { FILE_NEW = MSG_SUSER, 
			FILE_LOAD_MAP, 
			FILE_SAVE_MAP,
			FILE_SAVE_MAP_AS,
			FILE_LOAD_TILES, 
			FILE_QUIT, 
			FILE_MRUD0,
			FILE_MRUD1,
			FILE_MRUD2,
			FILE_MRUD3,
			EDIT_UNDO,
			EDIT_COPY, 
			EDIT_PASTE,
			EDIT_CUT,
			EDIT_DELETE,
			EDIT_RESIZE,
			TOOLS_BRUSH,
			TOOLS_FILL,
			TOOLS_SELECT,
			TOOLS_SCREENSHOT,
			HELP_HELP, 
			HELP_ABOUT
			};
		void MakeMenu();

		//engine stuff
		TEG_TILELIST *tilelist;
		DATAFILE *data;
		
		mrud current;
		mrudlist loaded;
		
		void mrud_menu_init(); // initialize mrud menu
		void add_current(); // add current file to mrud menu
		void load_desktop();
		void save_desktop();
		
		int menu_loadtiles ();
		int menu_resize ();
		int menu_new ();
		int menu_load ();
		int menu_pascal ();
		int menu_save ();
		int menu_save_as ();
		int menu_copy();
		int menu_paste();
		int menu_cut();
		int menu_delete();
		void menu_help ();
		void menu_selecttool (int tool);
		void menu_mrud(int item);
		void menu_about ();
		void menu_screenshot();
		void menu_undo();
		
		bool do_save ();
		bool do_save_as ();
		
		// data events
		void tileschanged();

	protected:
		virtual bool MsgClose();
		virtual void UpdateSize();
		virtual void HandleEvent(Widget &obj, int msg, int arg1, int arg2);
	public:
		MainDlg();
		virtual ~MainDlg();    
		void MapChanged(int code);    
};

#endif //MAIN_DLG
