//     :     :     :     :     :     :     :     :     :     :
//   --------:  ---------:  ---------:  ---------:  ----     :
// ..|      |*..|       |*..|       |*..|       |*..|  |*....:.
//   |      |*  |       |*  |       |*  |       |*  |  |*    :
//   ---  ---*  |  |-----*  |  |-----*  |  |-----*  |  |*    :
// ....|  |*.:..|  |--****..|  |-----*..|  |--****..|  |*....:.
//     |  |* :  |    |*  :  |  |--  |*  |    |*  :  |  |*    :
//     |  |* :  |  |--*  :  |  |*|  |*  |  |--*  :  |  |*    :
// ....|  |*.:..|  |-----:..|  ---  |*..|  |-----:..|  -----.:.
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
// ....----*.:..---------*..---------*..---------*..--------*:.
//     :**** :   *********   *********   *********   ********:
//     :     :     :     :     :     :     :     :     :     :
//
// Version 0.81
// Copyright (c) 2002-2007
// Martijn "Amarillion" van Iersel.
//
// This file is part of the Tegel tilemapping add-on for allegro
//
//    Tegel is free software; you can redistribute it and/or modify
//    it under the terms of the GNU Lesser General 
//    Public License as published by the Free Software 
//    Foundation; either version 2.1 of the License, or
//    (at your option) any later version.
//
//    Tegel is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public License
//    along with Tegel; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
// official website:
// http://tegel.sourceforge.net/
//
// I hope you like this program. Please send 
// questions, comments, flames and feature 
// requests to amarillion@yahoo.com.
//
/*
Scrollbox is a widget that can contain images larger than the area it has on the screen, it can use scrollbars
to accomodate those images.

ScrollBox uses a "merged" model, as opposed to the model found in ex03.cpp where the ScrollBox is split in two classes,
(say ScrollBox and ScrollClient) one handling the scrollbars and the other handling the image drawn.

I want classes derived from ScrollBox to be able to handle events, and with the split model, this means deriving
two classes, ScrollBox to handle events and ScrollClient to handle drawing.
*/

#ifndef MYSCROLLBOX_H
#define MYSCROLLBOX_H

#include <MASkinG.h>

class MyScrollBox;
    
// helper widget
// needed because the scroll area needs to be drawn after all the other scrollbox widgets are drawn.
// Dialog::draw() method called before all other widgets, so would be drawn over.
class MyScrollBoxHelper : public MAS::Widget
{
    MyScrollBox *parent;
    public:
        MyScrollBoxHelper(MyScrollBox *p);
        virtual void Draw(MAS::Bitmap &canvas);
};

// everything that has optional (h and/or v) scrollbars should be derived from this                
class MyScrollBox : public MAS::Dialog 
{
    private:
        MAS::Scroller hScroller, vScroller;
        MAS::ClearScreen clrscr;
        MAS::PanelSunken panel;
        MyScrollBoxHelper helper;
        //ScrollClient *client;
    
        int scrollWidth, scrollHeight; // size of the scroll area. If larger than the client area, 
        // scrollbars are displayed.    
        int scrollX, scrollY; // offset of the scroll area    
        int hopt, vopt;
        // scrollbars on, off or automatic.    
        int bufb, bufr, bufl, buft;
        // size of the panel edges    
        bool hashscroller, hasvscroller;        
        // helpers for determining client dimensions
        int GetHScrollerSize() { return hashscroller ? hScroller.h() : 0; }
        int GetVScrollerSize() { return hasvscroller ? vScroller.w() : 0; }
        // dimensions of the client canvas
    protected:
        virtual void DrawClientArea (MAS::Bitmap &dest, int xofst, int yofst) {}        
        void ResetScrollers ();
        void CheckPositionBounds();
	virtual void UpdateSize();        
	virtual void HandleEvent(MAS::Widget &obj, int msg, int arg1=0, int arg2=0);                        

        int GetClientBottom() { return h() - GetHScrollerSize() - bufb; }
        int GetClientRight() { return w() - GetVScrollerSize() - bufr; }
        int GetClientTop() { return buft; }
        int GetClientLeft() { return bufl; }
        int GetClientHeight() { return h() - GetHScrollerSize() - (buft + bufb); }
        int GetClientWidth() { return w() - GetVScrollerSize() - (bufl + bufr); }        
        
        bool IsInClientArea(MAS::Point &p);
    public:
        enum { SCROLLER_OFF, SCROLLER_ON, SCROLLER_AUTO };
        
        MyScrollBox();       
        virtual void HelperDraw(MAS::Bitmap &canvas);         
        virtual void MsgInitSkin();          
        // called whenever scrollbars change. Use it to update widgets that depend on the scroll position of this widget.
        virtual void MsgScroll() {}; 
        int GetXOffset() { return scrollX; }
        int GetYOffset() { return scrollY; }        
        void SetScrollSize (int width, int height);        
        void ScrollToArea (int x, int y, int w, int h);
        void SetPos (int x, int y);        
        void Center ();                             
        void SetScrollOption (int newhopt, int newvopt);
        int GetHScrollOption() { return hopt; }
        int GetVScrollOption() { return vopt; }        
};
#endif
