//     :     :     :     :     :     :     :     :     :     :
//   --------:  ---------:  ---------:  ---------:  ----     :
// ..|      |*..|       |*..|       |*..|       |*..|  |*....:.
//   |      |*  |       |*  |       |*  |       |*  |  |*    :
//   ---  ---*  |  |-----*  |  |-----*  |  |-----*  |  |*    :
// ....|  |*.:..|  |--****..|  |-----*..|  |--****..|  |*....:.
//     |  |* :  |    |*  :  |  |--  |*  |    |*  :  |  |*    :
//     |  |* :  |  |--*  :  |  |*|  |*  |  |--*  :  |  |*    :
// ....|  |*.:..|  |-----:..|  ---  |*..|  |-----:..|  -----.:.
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
// ....----*.:..---------*..---------*..---------*..--------*:.
//     :**** :   *********   *********   *********   ********:
//     :     :     :     :     :     :     :     :     :     :
//
// Version 0.81
// Copyright (c) 2002-2007
// Martijn "Amarillion" van Iersel.
//
// This file is part of the Tegel tilemapping add-on for allegro
//
//    Tegel is free software; you can redistribute it and/or modify
//    it under the terms of the GNU Lesser General 
//    Public License as published by the Free Software 
//    Foundation; either version 2.1 of the License, or
//    (at your option) any later version.
//
//    Tegel is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public License
//    along with Tegel; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
// official website:
// http://tegel.sourceforge.net/
//
// I hope you like this program. Please send 
// questions, comments, flames and feature 
// requests to amarillion@yahoo.com.
//
#ifndef RADARBOX_H
#define RADARBOX_H

#include "myscrollbox.h"
#include "tegel.h"
#include "mapbox.h"

#include <vector>

using namespace std;

class RadarBox;

class RadarBoxMapListener : public DataListener
{
    private:
        RadarBox *parent;
    public:
        RadarBoxMapListener (RadarBox *p)
        {
            assert (p);
            parent = p;
        }
        virtual void changed (int code);
};

class RadarBoxScrollListener : public DataListener
{
    private:
        RadarBox *parent;
    public:
        RadarBoxScrollListener (RadarBox *p)
        {
            assert (p);
            parent = p;
        }
        virtual void changed (int code);
};

class RadarBox : public MyScrollBox
{
    private:
        BITMAP *radar;
        int cursorx, cursory, cursorw, cursorh;
        vector<int> colors;
        void GetColors();
        TEG_MAP *map;
        MapWrapper *mapWrap;
        ScrollWrapper *scrollWrap;
        RadarBoxMapListener mapListener;
        RadarBoxScrollListener scrollListener;
        int magnification;
        void UpdateMagnification();
    public:
        RadarBox(MapWrapper *_mapWrap, ScrollWrapper *_scrollWrap);
        ~RadarBox();
        virtual void DrawClientArea (MAS::Bitmap &dest, int xofst, int yofst);
        virtual void MsgClick();
        virtual void UpdateSize();

        void MapChanged (int code);
        void ScrollChanged (int code);
};

#endif

