//     :     :     :     :     :     :     :     :     :     :
//   --------:  ---------:  ---------:  ---------:  ----     :
// ..|      |*..|       |*..|       |*..|       |*..|  |*....:.
//   |      |*  |       |*  |       |*  |       |*  |  |*    :
//   ---  ---*  |  |-----*  |  |-----*  |  |-----*  |  |*    :
// ....|  |*.:..|  |--****..|  |-----*..|  |--****..|  |*....:.
//     |  |* :  |    |*  :  |  |--  |*  |    |*  :  |  |*    :
//     |  |* :  |  |--*  :  |  |*|  |*  |  |--*  :  |  |*    :
// ....|  |*.:..|  |-----:..|  ---  |*..|  |-----:..|  -----.:.
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
// ....----*.:..---------*..---------*..---------*..--------*:.
//     :**** :   *********   *********   *********   ********:
//     :     :     :     :     :     :     :     :     :     :
//
// Version 0.81
// Copyright (c) 2002-2007
// Martijn "Amarillion" van Iersel.
//
// This file is part of the Tegel tilemapping add-on for allegro
//
//    Tegel is free software; you can redistribute it and/or modify
//    it under the terms of the GNU Lesser General 
//    Public License as published by the Free Software 
//    Foundation; either version 2.1 of the License, or
//    (at your option) any later version.
//
//    Tegel is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public License
//    along with Tegel; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
// official website:
// http://tegel.sourceforge.net/
//
// I hope you like this program. Please send 
// questions, comments, flames and feature 
// requests to amarillion@yahoo.com.
//
#include <assert.h>
#include <vector>
#include "flood.h"

using namespace std;

struct point
{
   int x, y;
   point *next;
};

typedef vector<point> stack;

point pop(stack *s)
{
    point result;
    result = (*s)[s->size()-1];
    s->pop_back();
    return result;
}

void push (stack *s, point p)
{
    s->push_back (p);
}

// returns number of tiles filled.
int teg_floodfill (TEG_MAP* map, int l, int x, int y, int filltile)
{
    int emptytile = teg_mapget (map, l, x, y);
    int count = 0;
    unsigned int maxstacksize = 0;
    assert (x >= 0);
    assert (y >= 0);
    assert (x < map->w);
    assert (y < map->h);
    assert (map != NULL);
    assert (l < map->dl);
    assert (l >= 0);
    assert (filltile != emptytile);

    stack points;
    point p;
    // add x, y to stack
    p.x = x;
    p.y = y;
    push (&points, p);

    // repeat until stack empty:
    while (!points.empty())
    {
        int left, right, y, x, flag, last;
        if (points.size() > maxstacksize) maxstacksize = points.size();
        p = pop (&points);
        y = p.y;
        left = p.x;
        right = p.x;
        // get point from stack. check if it is emptytile
        if (teg_mapget (map, l, left, y) != emptytile)
        {
            continue;
            // discard point
        }
        // scan & fill segment to left
        teg_mapput (map, l, left, y, filltile);
        count++;
        while (left > 0 && teg_mapget (map, l, left-1, y) == emptytile)
        {
            left--;
            teg_mapput (map, l, left, y, filltile);
            count++;
        }
        // scan & fill segment to right
        while (right < map->w-1 && teg_mapget (map, l, right+1, y) == emptytile)
        {
            right++;
            teg_mapput (map, l, right, y, filltile);
            count++;
        }
        // one above: start left. search for emptytile
        if (y > 0)
        {
            x = left;
            last = filltile;
            do
            {
                flag = teg_mapget (map, l, x, y-1);
                // search for first empty tile, then for first
                // empty tile after sequence of non-empty tile
                if (flag == emptytile && last != emptytile)
                {
                    // if found add to stack
                    p.x = x;
                    p.y = y-1;
                    push (&points, p);
                }
                last = flag;
                x++;
            }
            // until end of segment
            while (x <= right);
        }
        // repeat with one below
        if (y < map->h - 1)
        {
            x = left;
            last = filltile;
            do
            {
                flag = teg_mapget (map, l, x, y+1);
                // search for first empty tile, then for first
                // empty tile after sequence of non-empty tile
                if (flag == emptytile && last != emptytile)
                {
                    // if found add to stack
                    p.x = x;
                    p.y = y+1;
                    push (&points, p);
                }
                last = flag;
                x++;
            }
            // until end of segment
            while (x <= right);
        }
    }
    return count;
}

