//     :     :     :     :     :     :     :     :     :     :
//   --------:  ---------:  ---------:  ---------:  ----     :
// ..|      |*..|       |*..|       |*..|       |*..|  |*....:.
//   |      |*  |       |*  |       |*  |       |*  |  |*    :
//   ---  ---*  |  |-----*  |  |-----*  |  |-----*  |  |*    :
// ....|  |*.:..|  |--****..|  |-----*..|  |--****..|  |*....:.
//     |  |* :  |    |*  :  |  |--  |*  |    |*  :  |  |*    :
//     |  |* :  |  |--*  :  |  |*|  |*  |  |--*  :  |  |*    :
// ....|  |*.:..|  |-----:..|  ---  |*..|  |-----:..|  -----.:.
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
// ....----*.:..---------*..---------*..---------*..--------*:.
//     :**** :   *********   *********   *********   ********:
//     :     :     :     :     :     :     :     :     :     :
//
// Version 0.81
// Copyright (c) 2002-2007
// Martijn "Amarillion" van Iersel.
//
// This file is part of the Tegel tilemapping add-on for allegro
//
//    Tegel is free software; you can redistribute it and/or modify
//    it under the terms of the GNU Lesser General 
//    Public License as published by the Free Software 
//    Foundation; either version 2.1 of the License, or
//    (at your option) any later version.
//
//    Tegel is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public License
//    along with Tegel; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
// official website:
// http://tegel.sourceforge.net/
//
// I hope you like this program. Please send 
// questions, comments, flames and feature 
// requests to amarillion@yahoo.com.
//
#include "layerbox.h"
#include <string>

using namespace MAS;

LayerBox::LayerBox(LayerWrapper *_layerWrap) : layerListener (this)
{    
    layerWrap = _layerWrap;
    layerWrap->AddListener (&layerListener);
    SetRowNum (1);
    SetRowHeight (16);
    SetColNum(0);
    SetColWidth (32);    
}

LayerBox::~LayerBox()
{
    layerWrap->RemoveListener (&layerListener);
}

void LayerBox::MsgLPress(void)
{
    GridBox::MsgLPress();
    
    Point p = GetMousePos();
    
    int col = GetCol (p.x());
    int row = GetRow (p.y());
    
    if (col >= 0 && row == 0 && col < layerWrap->GetLayerNum())
    {
        layerWrap->SetActive (col);
    }
}

void LayerBox::MsgRPress(void)
{
    Point p = GetMousePos();
    
    int col = GetCol (p.x());
    int row = GetRow (p.y());
    
    if (col >= 0 && row == 0 && col < layerWrap->GetLayerNum())
    {
        layerWrap->InverseVisible (col);
    }
}

bool LayerBox::MsgXChar(int c)
{    
	// keys 1 .. 9 + Ctrl
    int col=(c>>8)-28;
    if (col >= 0 && col <= 8 && col < LayerBox::layerWrap->GetLayerNum())
    {
        if (!(key_shifts & KB_ALT_FLAG))
          layerWrap->SetActive (col);
        else
          layerWrap->InverseVisible (col);
		return true;
    }
    return false;
}

void LayerBox::MsgInitSkin()
{
    GridBox::MsgInitSkin();
    for (int i=0; i<4; i++) {
	SetFontColor(skin->fcol[Skin::INFO_BUTTON][i], skin->scol[Skin::INFO_BUTTON][i], i);
	SetFont(skin->fnt[Skin::INFO_BUTTON][i], i);
    }
    SetTextMode(-1);
    SetBitmap(Skin::BUTTON);
}

void LayerBox::DrawCell (Bitmap &dest, int col, int row, int x, int y)
{
    // get the button state
    int off;
    int state;
    bool down = layerWrap->IsVisible (col);
    bool active = (layerWrap->GetActiveLayer() == col);

    if (down) 
    {
        state = 1; 
        off = 3;
    } 
    else 
    {
        state = 0;
        off = 1;
    }

    // get the button colors and font
    Color fg = GetFontColor(state);
    Color bg = GetShadowColor(state);
    Font f = GetFont(state);
        
    char text[40]; 
    if (active)
    {
        sprintf (text, "%i*", col+1);
    }
    else
    {
        sprintf (text, "%i", col+1);
    }
    
    // Draw the button
    Bitmap bmp = GetBitmap();
    if (bmp) {
        bmp.MaskedTiledBlit(dest, 0, state, x, y, GetColWidth(), GetRowHeight(), 1, 4);
    }

    if (text) {
        // Calculate the position of the text		
        int text_x = GetColWidth()/2 - 1;
        int text_y = (GetRowHeight() - f.TextHeight())/2 - 2;
	
        // Displace the text a little when the button is down
        if (state == 1) {
            text_x += skin->buttonDisplacement;
            text_y += skin->buttonDisplacement;
        }
    
        // Print the text
        f.GUITextoutEx(dest, text, x+text_x, y+text_y, fg, bg, GetTextMode(), 2);
    }

}

void LayerBox::LayersChanged (int code)
{
    SetColNum (layerWrap->GetLayerNum());
    Redraw();
}
        
void LayerBoxLayerListener::changed (int code)
{
	parent->LayersChanged(code);
}
