//     :     :     :     :     :     :     :     :     :     :
//   --------:  ---------:  ---------:  ---------:  ----     :
// ..|      |*..|       |*..|       |*..|       |*..|  |*....:.
//   |      |*  |       |*  |       |*  |       |*  |  |*    :
//   ---  ---*  |  |-----*  |  |-----*  |  |-----*  |  |*    :
// ....|  |*.:..|  |--****..|  |-----*..|  |--****..|  |*....:.
//     |  |* :  |    |*  :  |  |--  |*  |    |*  :  |  |*    :
//     |  |* :  |  |--*  :  |  |*|  |*  |  |--*  :  |  |*    :
// ....|  |*.:..|  |-----:..|  ---  |*..|  |-----:..|  -----.:.
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
// ....----*.:..---------*..---------*..---------*..--------*:.
//     :**** :   *********   *********   *********   ********:
//     :     :     :     :     :     :     :     :     :     :
//
// Version 0.81
// Copyright (c) 2002-2007
// Martijn "Amarillion" van Iersel.
//
// This file is part of the Tegel tilemapping add-on for allegro
//
//    Tegel is free software; you can redistribute it and/or modify
//    it under the terms of the GNU Lesser General 
//    Public License as published by the Free Software 
//    Foundation; either version 2.1 of the License, or
//    (at your option) any later version.
//
//    Tegel is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public License
//    along with Tegel; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
// official website:
// http://tegel.sourceforge.net/
//
// I hope you like this program. Please send 
// questions, comments, flames and feature 
// requests to amarillion@yahoo.com.
//
#include "layerbox.h"
#include "mainDlg.h"
#include <assert.h>

#include "sizdlg.h"
#include "tegel.h"
#include <fstream>

#ifdef ALLEGRO_MINGW32
  #define snprintf _snprintf
#endif

void MainDlgMapListener::changed (int code)
{
    parent->MapChanged(code);
}

bool MainDlg::MsgClose() 
{
    Dialog::MsgClose();
	if (mapwrap.HasChanged())
	{
		MessageBox msg(
			"quit", 
			"Do you really want to quit?",
			"There are unsaved changes", 
			NULL, 
			"&Save", 
			"&Quit", 
			"&Cancel");
		switch (msg.Popup(this))
		{
			case 0: case 3: // cancel
				return false;
			case 1: // save
				if (do_save())
				{
					save_desktop();
					return true;
				}
				else // user cancelled save
				{
					return false;
				}
			case 2: // quit directly
				save_desktop();
				return true;
			default:
				assert (false); // shouldn't occur
				return false;
		}
	}
	return true;
}

void MainDlg::menu_mrud(int item)
{
    // load selected mrud item
    int error = false;
    
    if (item < 0 || item >= signed (loaded.size())) return; // out of bounds
        
    // first load the three objects in temporary vars
    DATAFILE *tempdata = NULL;
    TEG_TILELIST *temptiles = NULL;
    TEG_MAP *tempmap = NULL;

    if (!error)
    {
        tempdata = teg_load_datafile (loaded[item].datafile.c_str());
        if (!tempdata)
        {
            error = true;
            MessageBox msg("error", "Error during loading datafile",
                        loaded[item].datafile.c_str(), NULL, "&OK", NULL, NULL);
            msg.Popup(this);
        }
    }
    
    if (!error)
    {
        DATAFILE *di;

        di = tempdata;
        while (di->type != DAT_END && di->type != DAT_PALETTE)
            di++;

        if (di->type == DAT_PALETTE)
        {            
            set_palette ((RGB*)di->dat);
        }
        else
        {
            set_palette (default_palette);            
        }
        
        teg_unload_datafile (tempdata);
        tempdata = teg_load_datafile (loaded[item].datafile.c_str());

        if (!tempdata)
        {
            error = true;
            MessageBox msg("error", "Error during loading datafile",
                        loaded[item].datafile.c_str(), NULL, "&OK", NULL, NULL);
            msg.Popup(this);
        }
    }

    if (!error)
    {
        DATAFILE *datobj = find_datafile_object (tempdata, loaded[item].tilelist.c_str());        
        if (!datobj)        
        {
            error = true;
            MessageBox msg("error", "Tegel tilelist (TEGT) object not found:",
                        loaded[item].tilelist.c_str(), NULL, "&OK", NULL, NULL);
            msg.Popup(this);            
        }
        else
        {
            temptiles = (TEG_TILELIST*)(datobj->dat);
        }
    }

    if (!error)
    {
        tempmap = teg_loadmap (loaded[item].filename.c_str(), temptiles);
        if (!tempmap) 
        {
            error = true;
            MessageBox msg("error", "Error during loading tiles:",
                        teg_error, NULL, "&OK", NULL, NULL);
        }
    }

    if (!error && mapwrap.HasChanged()) 
    {
        MessageBox msg("overwrite", "Do you really want to load a map?",
                        "warning: current changes will be discarded", NULL, "&Yes", "&No", NULL);
        switch(msg.Popup(this)) {
            case 1:
                // ok, continue
            break;
            case 2:
                // not ok, keep current map
                error = true;
            break;
      };

    }
    
    if (error)
    {
        if (tempdata)
        {
            teg_unload_datafile(tempdata);
            tempdata = NULL;
        }
        if (tempmap)
        {
            teg_destroymap (tempmap);
            tempmap = NULL;
        }
    }
    if (!error)
    {
        if (data) teg_unload_datafile (data);
        data = tempdata;
        
        tilelist = temptiles;

        //mapchanged();        
        mapwrap.SetMap (tempmap);
		mapwrap.SetFile (loaded[item].filename, true, false);
        
        tileschanged();
        // make it the current mrud (move to top list)
        current.filename = loaded[item].filename;
        current.datafile = loaded[item].datafile;
        current.tilelist = loaded[item].tilelist;
        add_current();
    }    
}

void MainDlg::tileschanged()
{
    tilebox.setTiles (tilelist);    
}

void MainDlg::menu_screenshot()
{
    TEG_MAP *map = mapwrap.GetMap();
    if (map)
    {
        FileSelect dlg("Enter filename of mapshot", sscwd, 
            "Bitmap files (*.bmp)|PCX files (*.pcx)|All files (*.*)", FA_ARCH);
        char *sspath = dlg.Popup(this);
        if (sspath)
        {
            strcpy (sscwd, sspath);	
            replace_filename (sscwd, sscwd, "", sizeof(sscwd));
 
            BITMAP *ss = create_bitmap (
                map->tilelist->tilew * map->w,
                map->tilelist->tileh * map->h);
            int i;
            for (i = 0; i < map->dl; i++)
            {
                teg_draw (ss, map, i, 0, 0);
            }
            PALETTE pal;
            get_palette (pal);
            save_bitmap (sspath, ss, pal);
            destroy_bitmap (ss);
        }
    }
}

void MainDlg::menu_undo()
{
    mapwrap.Undo();
}

void MainDlg::menu_about()
{
    Window dlg;                    

    Button bOK;   
    PanelRaised theBox;                    // a raised panel
    TextArea ta;
     
    theBox.Shape (0,0,480,320);
    bOK.Setup (200,268,80,24, KEY_ENTER, D_EXIT, "&OK");
    ta.LoadLinesFromFile ("icons/about.txt");
    ta.Shape (20,20,440,240);
    
    // resize the dialog to the same size as the panel
    dlg.Resize(480, 320);
     
    // add the objects to the dialog
    dlg.Add(theBox);
    dlg.Add(bOK);
    dlg.Add(ta);

    // centre the dialog on the screen
    dlg.Centre();
     
    // execute the dialog
    dlg.Popup(this);
}

void MainDlg::menu_help()
{
    Window dlg;                     

    Button bOK;   
    PanelRaised theBox;                    // a raised panel
    TextArea ta;
    
    bOK.Setup (200,268,80,24, KEY_ENTER, D_EXIT, "&OK");
    theBox.Shape (0,0,480,320);
    ta.LoadLinesFromFile ("doc/tegel.txt");
    ta.Shape (20,20,440,240);
    
    // resize the dialog to the same size as the panel
    dlg.Resize(480, 320);
     
    // add the objects to the dialog
    dlg.Add(theBox);
    dlg.Add(bOK);
    dlg.Add(ta);
    
    // centre the dialog on the screen
    dlg.Centre();
     
    // execute the dialog
    dlg.Popup(this);
}

void MainDlg::menu_selecttool (int tool)
{
    if (tool >= 0 && tool <= 2)
    {    
        menuTools.Uncheck (0);
        menuTools.Uncheck (1);
        menuTools.Uncheck (2);
        menuTools.Check (tool);
        switch (tool)
        {
            
            case 0: mapbox.setTool (T_BRUSH); break;
            case 1: mapbox.setTool (T_FILL); break;
            case 2: mapbox.setTool (T_SELECT); break;
        }        
    }    
}

void MainDlg::MapChanged(int code)
{
	if (code == MapWrapper::EVT_CURSOR)
	{
		char buf [256];
		usprintf (buf, "x=%i y=%i", mapwrap.GetCursorX(), mapwrap.GetCursorY());
		lblCursor.SetText (buf);
	}
	else 
	{
		// set filename + change status
		std::string label = mapwrap.GetFilename();
		if (mapwrap.HasChanged()) { label += " *"; }
		lblFilename.SetText (label.c_str());
		
		// adjust menu items
		char *front = "Undo: ", *end = "\tCtrl-z";
		
		string temp = front;
		temp += mapwrap.GetUndoDescription();
		temp += end;
		
		menuEdit.GetItem (0)->SetText (temp.c_str());		
	}
}

// note that char*s returned by MASFileSelect.Popup() are still
// owned by the dialog and only exist as long as the dialog exists.
// make sure to copy the strings before you destroy the dialog objects.
int MainDlg::menu_loadtiles (void)
{
    DATAFILE *tempdata = NULL;
    TEG_TILELIST *temptiles = NULL;
    int error = FALSE;
    // step 1: load datafile    
    FileSelect dlg1("Select datafile containing tiles", cwd, 
        "Datafiles (*.dat)|All files (*.*)", FA_ARCH);
    char *datapath = NULL;
    string tileobj = "";
    datapath = dlg1.Popup(this);        
    
    // if the fileselector returned something, then setup the bitmap window
    if (datapath) 
    {
        set_color_conversion (COLORCONV_TOTAL);
        tempdata = teg_load_datafile (datapath);
        if (!tempdata)
        {
            error = true;
            MessageBox msg("error", "Error during loading datafile",
                        datapath, teg_error, "&OK", NULL, NULL);
            msg.Popup(this);
        }
        if (!error)
        {
            // change working dir
            strcpy (cwd, datapath);	
            replace_filename (cwd, cwd, "", sizeof(cwd));
            
            DATAFILE *di;
    
            di = tempdata;
            while (di->type != DAT_END && di->type != DAT_PALETTE)
                di++;

            if (di->type == DAT_PALETTE)
            {
                set_palette ((RGB*)di->dat);
            }
            else
            {
                set_palette (default_palette);            
            }
            
            teg_unload_datafile (tempdata);
            tempdata = teg_load_datafile (datapath);
            if (tempdata == NULL) 
            {
                error = TRUE;        
                MessageBox msg("error", "Error during loading datafile",
                        datapath, teg_error, "&OK", NULL, NULL);
                msg.Popup(this);
            }
        }        
    } 
    else 
    {
        error = TRUE;
        // user cancelled ...
    }
    
    if (!error)
    {
        // count number of TEGT objects in datafile
        int count = 0;
        DATAFILE *first = NULL;
        DATAFILE *di = tempdata;
        while (di->type != DAT_END)
        {
            if (di->type == DAT_TEG_TILELIST)
            {
                ++count;
                if (!first) first = di;
            }
            ++di;
        }
        if (count == 0)
        {
            MessageBox msg("error", "Datafile does not contain tilelist information",
                        datapath, NULL, "&OK", NULL, NULL);
            msg.Popup(this);
            error = true;
        } 
        else if (count == 1)
        {
            // easy, there is only one.
            temptiles = (TEG_TILELIST*)(first->dat);
            tileobj = get_datafile_property (first, DAT_ID('N','A','M','E'));
        } 
        else if (count > 1)
        {
            // now we have to ask the user.
            MessageBox msg("warning", "Warning: datafile contains more than one sets of tiles.",
                        "Selection feature not implemented yet.", datapath, "&OK", NULL, NULL);
            msg.Popup(this);
            // to be implemented...
            temptiles = (TEG_TILELIST*)(first->dat);
            tileobj = get_datafile_property (first, DAT_ID('N','A','M','E'));
        }
    }
    
    // if errors occured, remove them again.
    if (error)
    {
        if (tempdata)
        {
            teg_unload_datafile (tempdata);
            tempdata = NULL;
        }
    }
    else
    {        
        current.tilelist = tileobj; // mrud stuff
        current.datafile = datapath;
        
        RGB white = {255, 255, 255};
        // clean up possible previous tiles and data
        // and set the new ones.
        if (data)
        {
            teg_unload_datafile (data);
        }
        data = tempdata;
        // search for a palette and set it if found,
        // otherwise set default bios palette
        while (tempdata->type != DAT_END && tempdata->type != DAT_PALETTE)
            tempdata++;
        if (tempdata->type == DAT_PALETTE)
        {
            set_palette ((RGB*)tempdata->dat);
            set_color (255, &white);
            // now convert 8 bpp images in datafile
            // ....          
        }
        else
        {
            set_palette (default_palette);
            set_color (255, &white);
            
        }
        tilelist = temptiles;
        // redraw all
        tileschanged();
        mapwrap.SetMap (NULL);
		mapwrap.SetFile ("", false, false);
    }
    return D_O_K;
}

int MainDlg::menu_copy ()
{
	mapbox.Copy();
	return D_O_K;
}

int MainDlg::menu_paste ()
{
	mapbox.Paste();
	return D_O_K;
}

int MainDlg::menu_cut ()
{
	mapbox.Cut();
	return D_O_K;
}

int MainDlg::menu_delete ()
{
	mapbox.Delete();
	return D_O_K;
}

int MainDlg::menu_resize ()
{
    TEG_MAP *map = mapwrap.GetMap();
    if (map)
    {
        int w = map->w, h = map->h, dl = map->dl;
        if (SizeDlg::getMapSize (this, w, h, dl))
        {
            if (w > 0 && h > 0 && dl > 0)
            {
                mapwrap.Resize (dl, w, h);
            }
            else
            {
                MessageBox msg("Resize", "Error: you entered 0, or you didn't enter a number",
                        NULL, NULL, "&OK", NULL, NULL);
                msg.Popup(this);
            }
        }                
    }
    return D_O_K;
}

int MainDlg::menu_new (void)
{
    if (tilelist)
    {
		bool error = false;
		if (mapwrap.HasChanged()) 
		{
			MessageBox msg("new", "Do you really want to create a new map?",
							"warning: current changes will be discarded", NULL, "&Yes", "&No", NULL);
			switch(msg.Popup(this)) 
			{
				case 1:
					// ok, continue
				break;
				case 2:
					// not ok, keep current map
					error = true;
				break;
			}
		}
		if (!error)
		{
			int w = 64, h = 64, dl = 4;
			TEG_MAP* tempmap;
			if (SizeDlg::getMapSize (this, w, h, dl))
			{
				if (w > 0 && h > 0 && dl > 0)
				{
					tempmap = teg_createmap (dl, w, h, tilelist);
					if (tempmap != NULL) 
					{
						mapwrap.SetMap (tempmap);                
						mapwrap.SetFile ("untitled", false, false);
					}            
				}
				else
				{
					MessageBox msg("New Map", "Error: you entered 0, or you didn't enter a number",
							NULL, NULL, "&OK", NULL, NULL);
					msg.Popup(this);
				}
			}
		}
    }
    else
    {
        MessageBox msg("warning", "You should load a tilelist before you can create a new map!",
                        NULL, NULL, "&OK", NULL, NULL);
        msg.Popup(this);
    }
    return D_O_K;
}

int MainDlg::menu_load (void)
{
    TEG_MAP* tempmap = NULL;
    if (tilelist)
    {
		bool error = false;
		if (mapwrap.HasChanged()) 
		{
			MessageBox msg("load map", "Do you really want to load a map?",
							"warning: current changes will be discarded", NULL, "&Yes", "&No", NULL);
			switch(msg.Popup(this)) 
			{
				case 1:
					// ok, continue
				break;
				case 2:
					// not ok, keep current map
					error = true;
				break;
			}
		};
		if (!error)
		{
			// step 1: load datafile    
			FileSelect dlg("Select map file", cwd, 
				"Map files (*.map)|All files (*.*)", FA_ARCH);
			char *mappath = dlg.Popup(this);

			if (mappath)
			{
				tempmap = teg_loadmap (mappath, tilelist);
				if (tempmap)
				{
					current.filename = mappath; // mrud stuff
					add_current(); // mrud stuff
					mapwrap.SetMap (tempmap);
					mapwrap.SetFile (mappath, true, false);
					strcpy (cwd, mappath);	
					replace_filename (cwd, cwd, "", sizeof(cwd));
				}
			}
		}
    }
    else
    {
        MessageBox msg("warning", "You should load a tilelist before you can load a map!",
                        NULL, NULL, "&OK", NULL, NULL);
        msg.Popup(this);
    }
    return D_O_K;
}

int MainDlg::menu_save ()
{
	do_save();
	return D_O_K;
}

bool MainDlg::do_save()
{
    TEG_MAP *map = mapwrap.GetMap();
    if (map)
	{
		if (!mapwrap.IsTitled())
		{
			return do_save_as();
		}
		else
		{
            teg_savemap (map, mapwrap.GetFilename().c_str());
            strcpy (cwd, mapwrap.GetFilename().c_str());
            replace_filename (cwd, cwd, "", sizeof(cwd));
            current.filename = mapwrap.GetFilename(); // mrud stuff
            add_current();
			mapwrap.SetFile (mapwrap.GetFilename(), true, false);
		}
	}
	return true;
}

bool MainDlg::do_save_as()
{
	TEG_MAP *map = mapwrap.GetMap();
    if (map)
    {
		// step 1: load datafile    
        FileSelect dlg("Select map file", cwd, 
            "Map files (*.map)|All files (*.*)", FA_ARCH);
        char *mappath = dlg.Popup(this);
        if (mappath)
        {
            teg_savemap (map, mappath);
            strcpy (cwd, mappath);	
            replace_filename (cwd, cwd, "", sizeof(cwd));

            current.filename = mappath; // mrud stuff
            add_current();
			mapwrap.SetFile (mappath, true, false);
        }
		else
		{
			return false;
		}
    }
	return true;
}

int MainDlg::menu_save_as ()
{
    do_save_as();
    return D_O_K;
}

// called whenever mrud list changes
// adds mruds to file menu.
void MainDlg::mrud_menu_init()
{
    mrudlist::iterator i;
    int item;
    char buf[10];
    for (item = 0, i = loaded.begin(); i != loaded.end() && item < 4; i++, item++)
    {
        sprintf (buf, "%i: ", item + 1);
        string temp = buf;
        temp += get_filename (i->filename.c_str());
        menuFile.GetItem(9 + item)->SetText (temp.c_str());
    }
    for (; item < 4; item++)
    {
        sprintf (buf, "%i: ", item + 1);        
        menuFile.GetItem(9 + item)->SetText (buf);        
    }
}

// add current file to mrud list.
// check if the strings are not empty
// replace older entry with same map filename
void MainDlg::add_current()
{
    mrudlist::iterator i;

    // make sure map filename is unique
    for (i = loaded.begin(); i != loaded.end(); i++)
    {
        if (i->filename == current.filename)
        {
            loaded.erase (i);
            break;
        }
    }
    // add the current map
    loaded.push_front (current);
    // make sure size remains within limits
    while (loaded.size() > 8)
    {
        loaded.pop_back();
    }
    mrud_menu_init();
}

const char *desktop_file = "desktop.txt";

void MainDlg::save_desktop()
{
    ofstream outfile (desktop_file, ios::out);
    if (!outfile)
    {
        // file could not be created
    }
    else
    {
        outfile << cwd << endl;
        outfile << sscwd << endl;
        outfile << loaded.size() << endl;
        mrudlist::iterator i;
        for (i = loaded.begin(); i != loaded.end(); i++)
        {
            outfile << i->filename << endl
                << i->datafile << endl
                << i->tilelist << endl;
        }
        outfile.close();
    }
}

void MainDlg::load_desktop()
{
    ifstream infile (desktop_file, ios::in);
    if (!infile)
    {
        // no infile, read default settings 

        // change cwd to pwd.
        get_executable_name (cwd, sizeof(cwd));        
        replace_filename (cwd, cwd, "", sizeof(cwd));        
        get_executable_name (sscwd, sizeof(sscwd));
        replace_filename (sscwd, sscwd, "", sizeof(sscwd));        

        //clear mrud list
        loaded.clear();
    }
    else
    {
        int count = 0;
        mrud temp;
        char buf[360];
        infile.getline(cwd, sizeof (cwd));
        infile.getline(sscwd, sizeof (sscwd));
        infile >> count;
        infile.getline(buf, sizeof(buf));
        int i;
        for (i = 0; i < count; i++)
        {
           infile.getline (buf, sizeof(buf));
           temp.filename = buf;
           infile.getline (buf, sizeof(buf));
           temp.datafile = buf;
           infile.getline (buf, sizeof(buf));
           temp.tilelist = buf;
           loaded.push_back (temp);
        }
    }
    infile.close();
}


MainDlg::MainDlg() : Dialog(),
    mapwrap(),
    maplistener (this),
    layerwrap(),
    scrollwrap(),
    mapbox (&mapwrap, &layerwrap, &scrollwrap),
    radarbox (&mapwrap, &scrollwrap),
    layerbox (&layerwrap)
{
    mapwrap.AddListener (&maplistener);
    
    MakeMenu();
    load_desktop();
    mrud_menu_init();
    
    Add(desktop);
    
    pgup.Setup (KEY_PGUP);
    Add(pgup);
    pgdn.Setup (KEY_PGDN);
    Add(pgdn);
    ctrl_c.Setup (KEY_C, KB_CTRL_FLAG);
    Add(ctrl_c);
    ctrl_v.Setup (KEY_V, KB_CTRL_FLAG);
    Add(ctrl_v);
    ctrl_l.Setup (KEY_L, KB_CTRL_FLAG);
    Add(ctrl_l);
    ctrl_q.Setup (KEY_Q, KB_CTRL_FLAG);
    Add(ctrl_q);
    ctrl_n.Setup (KEY_N, KB_CTRL_FLAG);
    Add(ctrl_n);
    ctrl_s.Setup (KEY_S, KB_CTRL_FLAG);
    Add(ctrl_s);
    ctrl_z.Setup (KEY_Z, KB_CTRL_FLAG);
    Add(ctrl_z);
	ctrl_x.Setup (KEY_X, KB_CTRL_FLAG);
	Add(ctrl_x);
	del.Setup (KEY_DEL);
	Add(del);
    f1.Setup (KEY_F1);
    Add (f1);

    Add(menuMain);
    
	PALETTE palette;
	BITMAP *temp = load_bitmap ("icons/toolbar.bmp", palette);
	assert (temp);
	Bitmap toolbarImageMain = Bitmap(temp);
	assert (toolbarImageMain);

	const char *btn_tooltip[] = {
		"New", "Load", "Save", "Copy", "Paste", "Brush", "Flood Fill", "Select"
	};

	for (int i = 0; i < 8; ++i)
	{
		toolbar[i].Setup (320 + 32 * i, 0, 32, 32, 0, 0, "");
		toolbar[i].SetTooltipText (btn_tooltip[i]);
		toolbarImage[i] = Bitmap();
		toolbarImage[i].Create (24, 24);
		toolbarImageMain.Blit (toolbarImage[i],
			i * 24, 0, 0, 0, 24, 24);
		toolbar[i].SetButtonImage(toolbarImage[i]);
		Add (toolbar[i]);
	}

	char *fnRes[] = {
		"icons/mouse1.bmp", "icons/mouse2.bmp", "icons/mouse3.bmp", "icons/mouse4.bmp"
	};
	for (int i = 0; i < 4; ++i)
	{
		
		temp = load_bitmap (fnRes[i], palette);
		assert (temp);
		mouseCursor[i] = Bitmap (temp);
		assert (mouseCursor[i]);
	}

    data = NULL;
    tilelist = NULL;
        
    menu_selecttool (0);
    
    Add(radarbox);
    Add(mapbox);
    Add(tilebox);
    Add(layerbox);       
 
	lblFilename.SetText ("");
	lblFilename.ClearFlag (D_AUTOSIZE);
	Add (lblFilename);
	
	lblCursor.SetText ("");
	lblCursor.ClearFlag (D_AUTOSIZE);
	Add (lblCursor);

	lblLayerPalette.SetText ("Layer Palette:");
	Add (lblLayerPalette);
	
    tileschanged();        
}

MainDlg::~MainDlg() 
{
    teg_unload_datafile (data);    
}

void MainDlg::UpdateSize()
{
    desktop.Shape (0, 0, w(), h());
    
    mapbox.Shape (8, 32, w() - 184, h() - 104);
    
	lblLayerPalette.Shape (8, h() - 64, w() - 184, 16);
	layerbox.Shape (8, h() - 48, w() - 184, 24);
    
    tilebox.Shape (w() - 168, 32, 160, h() - 224);    
    radarbox.Shape (w() - 168, h() - 184, 160, 160);

	lblCursor.Shape (8, h() - 20, 240, 16);
	// note: TextMode gets reset if we do it earlier, e.g. in constructor.
	// this is a bug in MASkinG
	lblCursor.SetTextMode (GetSkin()->c_back); 
	
	lblFilename.Shape (248, h() - 20, 462, 16);
	lblFilename.SetTextMode (GetSkin()->c_back);	

}

void MainDlg::MakeMenu() 
{
    menuFile.Add("&New\tCtrl-N", FILE_NEW);
    menuFile.Add("&Load Map\tCtrl-L", FILE_LOAD_MAP);
    menuFile.Add("&Save Map\tCtrl-S", FILE_SAVE_MAP);
    menuFile.Add("Save Map &as", FILE_SAVE_MAP_AS);
    menuFile.Add();
    menuFile.Add("Load &Tiles\tCtrl-T", FILE_LOAD_TILES);
    menuFile.Add();
    menuFile.Add("&Quit\tCtrl-Q", FILE_QUIT);
    menuFile.Add();
    menuFile.Add("1: xxx", FILE_MRUD0);
    menuFile.Add("2: xxx", FILE_MRUD1);
    menuFile.Add("3: xxx", FILE_MRUD2);
    menuFile.Add("4: xxx", FILE_MRUD3);
    
    menuEdit.Add("Undo: xxx\tCtrl-z", EDIT_UNDO);
    menuEdit.Add("&Copy\tCtrl-C", EDIT_COPY);
    menuEdit.Add("&Paste\tCtrl-V", EDIT_PASTE);
	menuEdit.Add("Cu&t\tCtrl-X", EDIT_CUT);
	menuEdit.Add("Delete\tDel", EDIT_DELETE);
    menuEdit.Add();
    menuEdit.Add("&Resize", EDIT_RESIZE);
    
    menuTools.Add("&Brush", TOOLS_BRUSH);
    menuTools.Add("&Flood Fill", TOOLS_FILL);
    menuTools.Add("&Select rectangle", TOOLS_SELECT);
    menuTools.Add();
    menuTools.Add("&Map Screenshot", TOOLS_SCREENSHOT);
    
    menuHelp.Add("&Contents\tF1", HELP_HELP);
    menuHelp.Add("&About", HELP_ABOUT);
    
    menuMain.Add("&File", menuFile);
    menuMain.Add("&Edit", menuEdit);
    menuMain.Add("&Tools", menuTools);
    menuMain.Add("&Help", menuHelp);
}

void MainDlg::HandleEvent(Widget &obj, int msg, int arg1, int arg2)
{
#ifdef ALLEGRO_LINUX
    if (msg == MSG_IDLE)
    {
        struct timeval tv = {0, 1};
        select (0, NULL, NULL, NULL, &tv);        
    }
#endif    
    if (obj == tilebox && msg == MSG_TILE_CHANGED)
    {
        if (mouse_b & 2) mapbox.setCurrentTileR (arg1);
        else mapbox.setCurrentTile (arg1);
    }    
    if (obj == pgup && msg == MSG_ACTIVATE)
    {
        tilebox.selectPrev();
    }
    if (obj == pgdn && msg == MSG_ACTIVATE)
    {
        tilebox.selectNext();
    }
    if (obj == ctrl_q && msg == MSG_ACTIVATE)
    {
        Close();
    }    
    if ((obj == ctrl_c || obj == toolbar[TB_COPY]) && msg == MSG_ACTIVATE)
    {
        menu_copy();
    }    
    if ((obj == ctrl_v || obj == toolbar[TB_PASTE]) && msg == MSG_ACTIVATE)
    {
        menu_paste();
    }    
    if (obj == ctrl_x && msg == MSG_ACTIVATE)
    {
        menu_cut();
    }    
    if (obj == del && msg == MSG_ACTIVATE)
    {
        menu_delete();
    }    
    if ((obj == ctrl_n || obj == toolbar[TB_NEW]) && msg == MSG_ACTIVATE)
    {
        menu_new();
    }
    if ((obj == ctrl_s || obj == toolbar[TB_SAVE]) && msg == MSG_ACTIVATE)
    {
        menu_save();
    }
    if ((obj == ctrl_l || obj == toolbar[TB_OPEN]) && msg == MSG_ACTIVATE)
    {
        menu_load();
    }
    if (obj == ctrl_z && msg == MSG_ACTIVATE)
    {
        menu_undo();
    }
    if (obj == f1 && msg == MSG_ACTIVATE)
    {
        menu_help();
    }
	if (obj == toolbar[TB_BRUSH] && msg == MSG_ACTIVATE)
	{
		menu_selecttool (0);
	}
	if (obj == toolbar[TB_FILL] && msg == MSG_ACTIVATE)
	{
		menu_selecttool (1);
	}
	if (obj == toolbar[TB_SELECT] && msg == MSG_ACTIVATE)
	{
		menu_selecttool (2);
	}

    if (obj == menuMain) 
    {
       switch (msg) 
       {
          case FILE_NEW:
              menu_new();
              break;
          case FILE_LOAD_MAP:
              menu_load();
              break; 
          case FILE_SAVE_MAP:
              menu_save();
              break;
          case FILE_SAVE_MAP_AS:
              menu_save_as();
              break;
          case FILE_LOAD_TILES:
              menu_loadtiles();
              break; 
          case FILE_QUIT:
              Close();
              break; 
          case FILE_MRUD0:
              menu_mrud(0);
              break;
          case FILE_MRUD1:
              menu_mrud(1);
              break;
          case FILE_MRUD2:
              menu_mrud(2);
              break;
          case FILE_MRUD3:
              menu_mrud(3);
              break;
          case EDIT_UNDO:
              menu_undo();
              break;
          case EDIT_COPY:
			  menu_copy();
              break;
		  case EDIT_CUT:
			  menu_cut();
			  break;
		  case EDIT_DELETE:
			  menu_delete();
		      break;
          case EDIT_PASTE:
			  menu_paste();              
              break;
          case EDIT_RESIZE:
              menu_resize();
              break;
          case TOOLS_BRUSH:
              menu_selecttool(0);              
              break;
          case TOOLS_FILL:
              menu_selecttool(1);
              break;
          case TOOLS_SCREENSHOT:
              menu_screenshot();
              break;
          case TOOLS_SELECT:
              menu_selecttool(2);
              break;
          case HELP_HELP:
              menu_help();
              break;
          case HELP_ABOUT:
              menu_about();
              break;
          default:
              
              break;
       };
    }
}

