//     :     :     :     :     :     :     :     :     :     :
//   --------:  ---------:  ---------:  ---------:  ----     :
// ..|      |*..|       |*..|       |*..|       |*..|  |*....:.
//   |      |*  |       |*  |       |*  |       |*  |  |*    :
//   ---  ---*  |  |-----*  |  |-----*  |  |-----*  |  |*    :
// ....|  |*.:..|  |--****..|  |-----*..|  |--****..|  |*....:.
//     |  |* :  |    |*  :  |  |--  |*  |    |*  :  |  |*    :
//     |  |* :  |  |--*  :  |  |*|  |*  |  |--*  :  |  |*    :
// ....|  |*.:..|  |-----:..|  ---  |*..|  |-----:..|  -----.:.
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
// ....----*.:..---------*..---------*..---------*..--------*:.
//     :**** :   *********   *********   *********   ********:
//     :     :     :     :     :     :     :     :     :     :
//
// Version 0.81
// Copyright (c) 2002-2007
// Martijn "Amarillion" van Iersel.
//
// This file is part of the Tegel tilemapping add-on for allegro
//
//    Tegel is free software; you can redistribute it and/or modify
//    it under the terms of the GNU Lesser General 
//    Public License as published by the Free Software 
//    Foundation; either version 2.1 of the License, or
//    (at your option) any later version.
//
//    Tegel is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public License
//    along with Tegel; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
// official website:
// http://tegel.sourceforge.net/
//
// I hope you like this program. Please send 
// questions, comments, flames and feature 
// requests to amarillion@yahoo.com.
//
#include "myscrollbox.h"

using namespace MAS;

MyScrollBoxHelper::MyScrollBoxHelper (MyScrollBox *p) : parent (p)
{}
    
void MyScrollBoxHelper::Draw (Bitmap &canvas)
{
    if (parent) parent->HelperDraw (canvas);
}

// tries to make sure the area is visible
void MyScrollBox::ScrollToArea (int x, int y, int w, int h)
{    
    if (x + w > GetXOffset() + GetClientWidth())
        SetPos (x + w - GetClientWidth(), GetYOffset());
    else if (x < GetXOffset())
        SetPos (x, GetYOffset());
    if (y + h > GetYOffset() + GetClientHeight())
        SetPos (GetXOffset(), y + h - GetClientHeight());
    else if (y < GetYOffset())
        SetPos (GetXOffset(), y);   
}

bool MyScrollBox::IsInClientArea(Point &p)
{
    return (
        p.x() >= GetClientLeft() &&
        p.x() < GetClientRight() &&
        p.y() >= GetClientTop() &&
        p.y() < GetClientBottom()
    ); 
}

void MyScrollBox::MsgInitSkin()
{
    Dialog::MsgInitSkin();
    Bitmap panel = skin->GetBitmap (Skin::PANEL_SUNKEN);
    bufl = (panel.w() - panel.ThickX()) / 2;
    bufr = (panel.w() - panel.ThickX()) / 2;
    buft = (panel.h() - panel.ThickY()) / 2;
    bufb = (panel.h() - panel.ThickY()) / 2;   
}

// reset scrollers. called when
// scroll area changes in size (SetScrollSize)
// scroll options are changed 
// client area changes in size
void MyScrollBox::ResetScrollers ()
{
    hashscroller = false;
    hasvscroller = false;
    if (hopt == SCROLLER_AUTO)
    {                
        if (scrollWidth > w() - (bufl + bufr))                
            hashscroller = true;
        if  (scrollWidth > w() - (bufl + bufr) - vScroller.w() &&
                (vopt == SCROLLER_ON || 
                (vopt == SCROLLER_AUTO && scrollHeight > h() - (buft + bufb)))
            )
            hashscroller = true;
    }
    else
        hashscroller = (hopt == SCROLLER_ON);
    if (vopt == SCROLLER_AUTO)
    {                
        if (scrollHeight > h() - (buft + bufb))                
            hasvscroller = true;
        if  (scrollHeight > h() - (buft + bufb) - hScroller.h() &&
                (hopt == SCROLLER_ON || 
                (hopt == SCROLLER_AUTO && scrollWidth > w() - (bufl + bufr)))
            )
            hasvscroller = true;
    }
    else
        hasvscroller = (vopt == SCROLLER_ON);
    
    if (hashscroller)
    {
        hScroller.Unhide();
        hScroller.Shape (0, h() - hScroller.h(), w() - GetVScrollerSize(), hScroller.h());
        hScroller.SetRange (scrollWidth, GetClientWidth());            
        hScroller.SetPosition (scrollX);            
    }
    else
    {
        hScroller.Hide();
    }
    if (hasvscroller)
    {
        vScroller.Unhide();
        vScroller.Shape (w() - vScroller.w(), 0, vScroller.w(), h() - GetHScrollerSize());
        vScroller.SetRange (scrollHeight, GetClientHeight());
        vScroller.SetPosition (scrollY);
    }
    else
    {
        vScroller.Hide();
    }
    panel.Shape (0,0,w() - GetVScrollerSize(), h() - GetHScrollerSize());  
    helper.Shape (0,0,w() - GetVScrollerSize(), h() - GetHScrollerSize());            
    if (hashscroller && hasvscroller)
    {
        clrscr.Shape (w() - vScroller.w(), h() - hScroller.h(), vScroller.w(), hScroller.h());
        clrscr.Unhide();
    }
    else
    {
        clrscr.Hide();
    }                
    Redraw();            
}

// sets the offset of the client and scroller position in pixels
void MyScrollBox::SetPos (int x, int y)
{
    if (scrollX != x || scrollY != y)
    {
        scrollX = x;
        scrollY = y;
        CheckPositionBounds();                
        Redraw();
    }           
}; 

// center view        
void MyScrollBox::Center ()
{
    SetPos (
        (scrollWidth - GetClientWidth()) / 2,
        (scrollHeight - GetClientHeight()) / 2);
}

// set options for scrollers, default = auto
void MyScrollBox::SetScrollOption (int newhopt, int newvopt)
{
    if (newhopt != hopt || newvopt != vopt)
    {
        hopt = newhopt;
        vopt = newvopt;
        ResetScrollers();
    }
}

MyScrollBox::MyScrollBox()  : Dialog(), helper (this),
    scrollWidth (0), scrollHeight (0), scrollX (0), scrollY (0), 
    hopt (SCROLLER_AUTO), vopt (SCROLLER_AUTO), hashscroller (false), hasvscroller (false)
{
    Add(clrscr);
    hScroller.SetOrientation(1);
    vScroller.SetOrientation(0);
    hScroller.Hide();
    vScroller.Hide();
    hScroller.SetIncrement (16);
    vScroller.SetIncrement (16);
    Add(hScroller);
    Add(vScroller);            
    Add(panel);
    Add(helper);
    bufl = 0;
    bufr = 0;
    buft = 0;
    bufb = 0;
}

void MyScrollBox::SetScrollSize (int width, int height)
        {
            if (scrollWidth != width || scrollHeight != height)
            {
                scrollWidth = width;
                scrollHeight = height;
                CheckPositionBounds();            
                ResetScrollers ();
            }
        }
        
// make sure the positions scrollx and scrolly are between 0 and scrollWidth / scrollHeight.        
void MyScrollBox::CheckPositionBounds()
        {
            if (scrollX >= scrollWidth - GetClientWidth()) 
                scrollX = scrollWidth - GetClientWidth() - 1;
            if (scrollX < 0) 
                scrollX = 0;
            if (scrollY >= scrollHeight - GetClientHeight()) 
                scrollY = scrollHeight - GetClientHeight() - 1;
            if (scrollY < 0) 
                scrollY = 0;
            hScroller.SetPosition (scrollX);
            vScroller.SetPosition (scrollY);            
        }
        
void MyScrollBox::UpdateSize()
        {            
            Dialog::UpdateSize();

            // size of parent dialog has changed.
            // see if scrollers need to be added / removed.
            CheckPositionBounds();
            ResetScrollers();
	}
	
void MyScrollBox::HandleEvent(Widget &obj, int msg, int arg1, int arg2) 
        {
            if (msg == MSG_SCROLL)
            {
		if (obj == hScroller) 
                {
                    scrollX = arg1;
                    Redraw();
                }
                else if (obj == vScroller) 
                {
                    scrollY = arg1;
                    Redraw();
                }
                //GetParent()->HandleEvent(*this, MSG_SCROLL);
                MsgScroll();
            }		
            Dialog::HandleEvent(obj, msg, arg1, arg2);
	}

// helperdraw is different from normal draw:
// normal draw is called before subwidgets
// helperdraw is called after subwidgets.
void MyScrollBox::HelperDraw(Bitmap &canvas)
        {            
            Bitmap client;
            client.CreateSubBitmap (canvas, 
                GetClientLeft(), GetClientTop(), GetClientWidth(), GetClientHeight());
			client.SetClip (canvas.GetClip());
            DrawClientArea (client, scrollX, scrollY);
            client.Destroy();
        }
