//     :     :     :     :     :     :     :     :     :     :
//   --------:  ---------:  ---------:  ---------:  ----     :
// ..|      |*..|       |*..|       |*..|       |*..|  |*....:.
//   |      |*  |       |*  |       |*  |       |*  |  |*    :
//   ---  ---*  |  |-----*  |  |-----*  |  |-----*  |  |*    :
// ....|  |*.:..|  |--****..|  |-----*..|  |--****..|  |*....:.
//     |  |* :  |    |*  :  |  |--  |*  |    |*  :  |  |*    :
//     |  |* :  |  |--*  :  |  |*|  |*  |  |--*  :  |  |*    :
// ....|  |*.:..|  |-----:..|  ---  |*..|  |-----:..|  -----.:.
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
// ....----*.:..---------*..---------*..---------*..--------*:.
//     :**** :   *********   *********   *********   ********:
//     :     :     :     :     :     :     :     :     :     :
//
// Version 0.81
// Copyright (c) 2002-2007
// Martijn "Amarillion" van Iersel.
//
// This file is part of the Tegel tilemapping add-on for allegro
//
//    Tegel is free software; you can redistribute it and/or modify
//    it under the terms of the GNU Lesser General 
//    Public License as published by the Free Software 
//    Foundation; either version 2.1 of the License, or
//    (at your option) any later version.
//
//    Tegel is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public License
//    along with Tegel; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
// official website:
// http://tegel.sourceforge.net/
//
// I hope you like this program. Please send 
// questions, comments, flames and feature 
// requests to amarillion@yahoo.com.
//
#include "tilebox.h"
#include <assert.h>
#include <allegro.h>

using namespace MAS;

TileBox::TileBox()
{
    tiles = NULL;
    timerId = -1;
    frame = 0;
}

TileBox::~TileBox()
{
}

void TileBox::MsgTimer(int t)
{
    if (tiles && t == timerId && t > 0)
    {
        ++frame;
        if (frame >= tiles->animsteps)
            frame = 0;
        Redraw();        
    }
}
        

void TileBox::MsgClick(void)
{
    Point p = GetMousePos();
    int col = GetCol (p.x());
    int row = GetRow (p.y());
    int index = GetColNum() * row + col;
    if (col < GetColNum()) {
	if (mouse_b & 2) setSelectedRMB (index);
	else setSelected (index);
    }
}

void TileBox::DrawCell (Bitmap &dest, int col, int row, int x, int y)
{
    if (!tiles)
        return;
    
    int index = GetColNum() * row + col;
    
    if (index >= 0 && index < tiles->tilenum)
    {
        if (index == selected)
        {
            rectfill (dest, x, y ,x + GetColWidth(), y + GetRowHeight(), makecol (0,0,255));
        }

        if (index == selected_r)
        {
            rectfill (dest, x+1, y+1 ,x + GetColWidth()-1, y + GetRowHeight()+-1, makecol (0,255,0));
        }

        if (frame >= 0 && frame < tiles->animsteps)
        {
            index += frame * tiles->tilenum;
        }        
        teg_drawtile (dest, tiles, index, x + 2, y + 2);        
    }    
}

void TileBox::setTiles (TEG_TILELIST *value)
{
    // tiles that fit on a row
    tiles = value;
    
    if (!tiles) 
    {
        SetColNum (1);
        SetRowNum (1);
        SetColWidth (16);
        SetRowHeight (16);
        return;
    }
    
    int colw, rowh;
    
    colw = tiles->tilew + 4;
    rowh = tiles->tileh + 4;
    
    SetScrollOption (SCROLLER_OFF, SCROLLER_OFF);
    
    int tiles_per_row = GetClientWidth() / colw;
    if (tiles_per_row < 1) tiles_per_row = 1;
    
    SetScrollOption (SCROLLER_OFF, SCROLLER_ON);
        
    if (tiles_per_row > 1 && tiles_per_row * colw > GetClientWidth())
        tiles_per_row--;
    
    SetScrollOption (SCROLLER_OFF, SCROLLER_AUTO);
    
    SetColWidth (colw);
    SetRowHeight (rowh);
    SetColNum (tiles_per_row);
    SetRowNum ((tiles->tilenum + tiles_per_row - 1) / tiles_per_row);
    
    setSelected (0);
    setSelectedRMB (0);
    
    if (timerId > 0) 
    {
        Timer::Kill (timerId);
        timerId = -1;
    }
    if (tiles->animsteps > 0) 
    {
        timerId = Timer::Install (300);
    }    
}

void TileBox::selectNext()
{
    if (tiles)
    {
        if (selected < tiles->tilenum - 1)
        {
            setSelected (selected+1);
        }
        else
        {
            setSelected (0);
        }        
    }
    
}

void TileBox::selectPrev()
{
    if (tiles)
    {
        if (selected > 0)
        {
            setSelected (selected - 1);
        }
        else
        {
            setSelected (tiles->tilenum-1);
        }
    }                
}

void TileBox::setSelected(int value)
{
    if (value != selected)
    {
        selected = value;
        GetParent()->HandleEvent( *this, MSG_TILE_CHANGED, selected);
        
        // make sure selected tile is visible.
        // scroll to new position if necessary.
        ScrollToArea (0, GetRowHeight() * (selected / GetColNum()), 
            GetColWidth(), GetRowHeight());
        Redraw();
    }   
}

void TileBox::setSelectedRMB(int value)
{
    if (value != selected_r)
    {
        selected_r = value;
        GetParent()->HandleEvent( *this, MSG_TILE_CHANGED, selected_r);
        
        // make sure selected tile is visible.
        // scroll to new position if necessary.
        ScrollToArea (0, GetRowHeight() * (selected_r / GetColNum()), 
            GetColWidth(), GetRowHeight());
        Redraw();
    }   
}


