//     :     :     :     :     :     :     :     :     :     :
//   --------:  ---------:  ---------:  ---------:  ----     :
// ..|      |*..|       |*..|       |*..|       |*..|  |*....:.
//   |      |*  |       |*  |       |*  |       |*  |  |*    :
//   ---  ---*  |  |-----*  |  |-----*  |  |-----*  |  |*    :
// ....|  |*.:..|  |--****..|  |-----*..|  |--****..|  |*....:.
//     |  |* :  |    |*  :  |  |--  |*  |    |*  :  |  |*    :
//     |  |* :  |  |--*  :  |  |*|  |*  |  |--*  :  |  |*    :
// ....|  |*.:..|  |-----:..|  ---  |*..|  |-----:..|  -----.:.
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
// ....----*.:..---------*..---------*..---------*..--------*:.
//     :**** :   *********   *********   *********   ********:
//     :     :     :     :     :     :     :     :     :     :
//
// Version 0.81
// Copyright (c) 2002-2007
// Martijn "Amarillion" van Iersel.
//
// This file is part of the Tegel tilemapping add-on for allegro
//
//    Tegel is free software; you can redistribute it and/or modify
//    it under the terms of the GNU Lesser General 
//    Public License as published by the Free Software 
//    Foundation; either version 2.1 of the License, or
//    (at your option) any later version.
//
//    Tegel is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public License
//    along with Tegel; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
// official website:
// http://tegel.sourceforge.net/
//
// I hope you like this program. Please send 
// questions, comments, flames and feature 
// requests to amarillion@yahoo.com.
//
#include "undo.h"
#include "tegel.h"
#include "mapbox.h"

using namespace std;

UndoList::UndoList () : items()
{
}

UndoList::~UndoList()
{
    Clear();
}

// remove all UndoItems from the list
void UndoList::Clear()
{
    std::vector<UndoItem*>::iterator i;
    for (i = items.begin(); i != items.end(); ++i)
    {
        delete *i;
    }
    items.clear();
}

void UndoList::Undo(MapWrapper *mw)
{
    if (items.size() > 0)
    {
        UndoItem *i = items[items.size() - 1];
        i->Undo (mw);
        delete i;
        items.pop_back();
    }
}

// UndoList itself takes care of cleaning up memory after item is no longer needed
void UndoList::Add (UndoItem *i)
{
    items.push_back (i);
}

string UndoList::GetDescription()
{ 
    if (items.size() > 0) 
        return items.back()->GetDescription(); 
    else 
        return "-"; 
}

UndoBrush::UndoBrush (int _l, int _x, int _y, int _oldtile) : x(_x), y(_y), l(_l), oldtile(_oldtile)
{
    description = "tile brush";
}

void UndoBrush::Undo(MapWrapper *mw)
{
    teg_mapput (mw->map, l, x, y, oldtile);
	mw->SetCursor (x, y);
}

// stores a copy of map!
UndoTotal::UndoTotal (TEG_MAP *map, string _description, int _x, int _y) : x(_x), y (_y)
{
    oldmap = teg_create_copy (map);
    description = _description;
}

void UndoTotal::Undo(MapWrapper *mw)
{
    teg_destroymap (mw->map);
    mw->map = oldmap;
	mw->SetCursor (x, y);
    oldmap = NULL;
}

UndoTotal::~UndoTotal ()
{
    if (oldmap) teg_destroymap (oldmap);
}
