//     :     :     :     :     :     :     :     :     :     :
//   --------:  ---------:  ---------:  ---------:  ----     :
// ..|      |*..|       |*..|       |*..|       |*..|  |*....:.
//   |      |*  |       |*  |       |*  |       |*  |  |*    :
//   ---  ---*  |  |-----*  |  |-----*  |  |-----*  |  |*    :
// ....|  |*.:..|  |--****..|  |-----*..|  |--****..|  |*....:.
//     |  |* :  |    |*  :  |  |--  |*  |    |*  :  |  |*    :
//     |  |* :  |  |--*  :  |  |*|  |*  |  |--*  :  |  |*    :
// ....|  |*.:..|  |-----:..|  ---  |*..|  |-----:..|  -----.:.
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
// ....----*.:..---------*..---------*..---------*..--------*:.
//     :**** :   *********   *********   *********   ********:
//     :     :     :     :     :     :     :     :     :     :
//
// Version 0.81
// Copyright (c) 2002-2007
// Martijn "Amarillion" van Iersel.
//
// This file is part of the Tegel tilemapping add-on for allegro
//
//    Tegel is free software; you can redistribute it and/or modify
//    it under the terms of the GNU Lesser General 
//    Public License as published by the Free Software 
//    Foundation; either version 2.1 of the License, or
//    (at your option) any later version.
//
//    Tegel is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public License
//    along with Tegel; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
// official website:
// http://tegel.sourceforge.net/
//
// I hope you like this program. Please send 
// questions, comments, flames and feature 
// requests to amarillion@yahoo.com.
//
#include "tegel.h"

DATAFILE *data;
TEG_TILELIST *tilelist;
TEG_MAP *map;

void test_anim ()
{
    BITMAP *buffer;
    int quit = FALSE;
    int x = 0, y = 0;
    int frame = 0;

    // to avoid flicker the program makes use of a double buffering system
    buffer = create_bitmap (screen->w, screen->h);
    // create a 64x64 tile bitmap and draw something on it.
    text_mode (-1);

    while (!quit)
    {
        // act on keyboard input
        if (key[KEY_ESC]) quit = TRUE;
        if (key[KEY_UP] && y > 0)
            y--;
        if (key[KEY_DOWN] && y < teg_pixelh(map) - SCREEN_H)
            y++;
        if (key[KEY_LEFT] && x > 0)
            x--;
        if (key[KEY_RIGHT] && x < teg_pixelw(map) - SCREEN_W)
            x++;

	teg_draw_frame (buffer, map, 0, x, y, frame);
	teg_draw_frame (buffer, map, 1, x, y, frame);
	if (++frame >= map->tilelist->animsteps) frame = 0;
        vsync();
        blit (buffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);

    }
    destroy_bitmap (buffer);
}

int main ()
{
    // initialise allegro
    if (allegro_init () < 0)
    {
        allegro_message ("Error: Could not initialise allegro");
        return -1;
    }
    // initialise gfx mode
    set_color_depth(16);
    if (set_gfx_mode (GFX_AUTODETECT, 320, 200, 0, 0) < 0)
    {
        allegro_message ("Error: Could not set graphics mode");
        return -1;
    }
    // initialise keyboard
    install_keyboard ();
    clear_keybuf ();
	teg_install();

    data = teg_load_datafile ("../data//fractal.dat");
    tilelist = find_datafile_object (data, "FRACTAL")->dat;
    map = teg_loadmap ("../data//fractal1.map", tilelist);

    // call the example function
    test_anim ();

    teg_uninstall();
    // exit allegro
	allegro_exit ();

    return 0;

} END_OF_MAIN ();

