//     :     :     :     :     :     :     :     :     :     :
//   --------:  ---------:  ---------:  ---------:  ----     :
// ..|      |*..|       |*..|       |*..|       |*..|  |*....:.
//   |      |*  |       |*  |       |*  |       |*  |  |*    :
//   ---  ---*  |  |-----*  |  |-----*  |  |-----*  |  |*    :
// ....|  |*.:..|  |--****..|  |-----*..|  |--****..|  |*....:.
//     |  |* :  |    |*  :  |  |--  |*  |    |*  :  |  |*    :
//     |  |* :  |  |--*  :  |  |*|  |*  |  |--*  :  |  |*    :
// ....|  |*.:..|  |-----:..|  ---  |*..|  |-----:..|  -----.:.
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
// ....----*.:..---------*..---------*..---------*..--------*:.
//     :**** :   *********   *********   *********   ********:
//     :     :     :     :     :     :     :     :     :     :
//
// Version 0.81
// Copyright (c) 2002-2007
// Martijn "Amarillion" van Iersel.
//
// This file is part of the Tegel tilemapping add-on for allegro
//
//    Tegel is free software; you can redistribute it and/or modify
//    it under the terms of the GNU Lesser General 
//    Public License as published by the Free Software 
//    Foundation; either version 2.1 of the License, or
//    (at your option) any later version.
//
//    Tegel is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public License
//    along with Tegel; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
// official website:
// http://tegel.sourceforge.net/
//
// I hope you like this program. Please send 
// questions, comments, flames and feature 
// requests to amarillion@yahoo.com.
//
#include "tegel.h"
#include <assert.h>

void run()
{
    DATAFILE *data, *data2;
    TEG_TILELIST *tilelist;
    TEG_MAP *map;

    teg_install();
    
    //method 1
    //tiles in datafile, tilelist and maps in separate files.
    //to be used while you are still developping maps and tilelists.
    //this method is going to be depracated when the editor can create/edit tilelists in place in the
    //datafile
    
    data = load_datafile ("../data/method1.dat");
    data2 = find_datafile_object (data, "PAL_GAME");
    set_palette ((RGB*)(data2->dat));
        
    // teg_loadtiles is going to be depracated
    tilelist = teg_loadtiles ("../data/method1.tll", data);
    if (!tilelist) allegro_message ("error : %s\n", teg_error);
    // teg_loadmap is going to be depracated, instead use teg_loadmap from method #2.
    map = teg_loadmap ("../data/method1.map", tilelist);
    
    teg_draw (screen, map, 0, 0, 0);
    textout_ex (screen, font, "method 1", 0, 0, YELLOW, -1);
    textout_ex (screen, font, "tile bitmaps:", 0, 8, WHITE, -1);
    textout_ex (screen, font, "tilelist:", 0, 16, WHITE, -1);
    textout_ex (screen, font, "map:", 0, 24, WHITE, -1);
    textout_ex (screen, font, "../data/method1.dat", 128, 8, CYAN, -1);
    textout_ex (screen, font, "../data/method1.tll", 128, 16, CYAN, -1);
    textout_ex (screen, font, "../data/method1.map", 128, 24, CYAN, -1);
    
    readkey();
    
    teg_destroymap (map);    
    // teg_destroytiles is going to be obsolete
    teg_destroytiles (tilelist);    
    unload_datafile (data);
    
    
    //method 2
    //maps in separate files
    //to be used while you are still developping maps.
    
    data = teg_load_datafile ("../data/method2.dat");    
    data2 = find_datafile_object (data, "METHOD2");    
    if (!data2) allegro_message ("error : %s\n", teg_error);
    
    map = teg_loadmap ("../data/method2.map", (TEG_TILELIST*)(data2->dat));
    if (!map) allegro_message ("error : %s\n", teg_error);
    
    teg_draw (screen, map, 0, 0, 0);
    textout_ex (screen, font, "method 2", 0, 0, YELLOW, -1);
    textout_ex (screen, font, "tile bitmaps:", 0, 8, WHITE, -1);
    textout_ex (screen, font, "tilelist:", 0, 16, WHITE, -1);
    textout_ex (screen, font, "map:", 0, 24, WHITE, -1);
    textout_ex (screen, font, "../data/method2.dat", 128, 8, CYAN, -1);
    textout_ex (screen, font, "../data/method2.dat", 128, 16, CYAN, -1);
    textout_ex (screen, font, "../data/method2.map", 128, 24, CYAN, -1);
    
    readkey();
    
    teg_destroymap (map);
    teg_unload_datafile (data);
    
    
    // method 3
    // everything together in a single datafile
    // preferred method for release versions of not too large games
    
    data = teg_load_datafile ("../data/method3.dat");
    map = (TEG_MAP*)((find_datafile_object (data, "MYMAP"))->dat);
    map->tilelist = (TEG_TILELIST*)((find_datafile_object(data, "MYTILES"))->dat);
    
    teg_draw (screen, map, 0, 0, 0);
    textout_ex (screen, font, "method 3", 0, 0, YELLOW, -1);
    textout_ex (screen, font, "tile bitmaps:", 0, 8, WHITE, -1);
    textout_ex (screen, font, "tilelist:", 0, 16, WHITE, -1);
    textout_ex (screen, font, "map:", 0, 24, WHITE, -1);
    textout_ex (screen, font, "../data/method3.dat", 128, 8, CYAN, -1);
    textout_ex (screen, font, "../data/method3.dat", 128, 16, CYAN, -1);
    textout_ex (screen, font, "../data/method3.dat", 128, 24, CYAN, -1);
    
    readkey();
    
    teg_unload_datafile (data);
        
    teg_uninstall();
}


int main ()
{
    // initialise allegro
    if (allegro_init () < 0)
    {
        allegro_message ("Error: Could not initialise allegro");
        return -1;
    }
    // initialise gfx mode
    if (set_gfx_mode (GFX_AUTODETECT, 320, 200, 0, 0) < 0)
    {
        allegro_message ("Error: Could not set graphics mode");
        return -1;
    }
    // initialise keyboard
    install_keyboard ();
    clear_keybuf ();

    // call the example function
    run ();

    // exit allegro
    allegro_exit ();

    return 0;

} END_OF_MAIN ();

