//     :     :     :     :     :     :     :     :     :     :
//   --------:  ---------:  ---------:  ---------:  ----     :
// ..|      |*..|       |*..|       |*..|       |*..|  |*....:.
//   |      |*  |       |*  |       |*  |       |*  |  |*    :
//   ---  ---*  |  |-----*  |  |-----*  |  |-----*  |  |*    :
// ....|  |*.:..|  |--****..|  |-----*..|  |--****..|  |*....:.
//     |  |* :  |    |*  :  |  |--  |*  |    |*  :  |  |*    :
//     |  |* :  |  |--*  :  |  |*|  |*  |  |--*  :  |  |*    :
// ....|  |*.:..|  |-----:..|  ---  |*..|  |-----:..|  -----.:.
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
// ....----*.:..---------*..---------*..---------*..--------*:.
//     :**** :   *********   *********   *********   ********:
//     :     :     :     :     :     :     :     :     :     :
//
// Version 0.81
// Copyright (c) 2002-2007
// Martijn "Amarillion" van Iersel.
//
// This file is part of the Tegel tilemapping add-on for allegro
//
//    Tegel is free software; you can redistribute it and/or modify
//    it under the terms of the GNU Lesser General 
//    Public License as published by the Free Software 
//    Foundation; either version 2.1 of the License, or
//    (at your option) any later version.
//
//    Tegel is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public License
//    along with Tegel; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
// official website:
// http://tegel.sourceforge.net/
//
// I hope you like this program. Please send 
// questions, comments, flames and feature 
// requests to amarillion@yahoo.com.
//
#include "tegel.h"

DATAFILE *data;
TEG_TILELIST *tilelist;
TEG_MAP *map;


void memory_putpixel(BITMAP *bmp, int x, int y, int color)
{
   bmp->line[y][x] = color;
}

void memory_putpixel_15_or_16_bpp(BITMAP *bmp, int x, int y, int color)
{
   ((short *)bmp->line[y])[x] = color;
}

void memory_putpixel_32(BITMAP *bmp, int x, int y, int color)
{
   ((long *)bmp->line[y])[x] = color;
}

int memory_getpixel(BITMAP *bmp, int x, int y)
{
   return bmp->line[y][x];
}

int memory_getpixel_15_or_16_bpp(BITMAP *bmp, int x, int y)
{
   return ((short *)bmp->line[y])[x];
}

int memory_getpixel_32(BITMAP *bmp, int x, int y)
{
   return ((long *)bmp->line[y])[x];
}

/*
draw a rotated map.
prerequisites:
   - map is large or largei
   - map->w, map->h, tilew, tileh are powers of 2
*/
void teg_masked_rot_map (BITMAP *bmp, TEG_MAP *map, fixed angle, fixed cx, fixed cy, int frame)
{
    // current screen position
    int screen_x, screen_y;
    
    void (*ppfunc)(BITMAP *, int, int, int);
    int (*gpfunc)(BITMAP *, int, int);
    
    TEG_TILELIST *tiles = map->tilelist;
    BITMAP *tile = map->tilelist->largebmp;

    int tiles_per_row = tile->w / tiles->tilew;

    // masks to make sure we don't read pixels outside the tile
    int tile_mask_x = (tiles->tilew - 1);
    int tile_mask_y = (tiles->tileh -1);
    int map_mask_x = (map->w - 1);
    int map_mask_y = (map->h - 1);
    int tilew_log = 5; // !!!!!!!!!!!!
    int tileh_log = 5;
    int mask_color = makecol (255, 0, 255);

    // step for points in space between two pixels on a horizontal line
    fixed line_dx, line_dy;

    // current space position
    fixed space_x, space_y;
    fixed old_space_x, old_space_y;

    TEG_TILE *frametiles = map->tilelist->tiles + frame * map->tilelist->tilenum;
    
    // calculate the dx and dy of points in space when we step
    // through all points on this line.
    line_dx = -fsin(angle);
    line_dy = fcos(angle);

    // calculate the starting position
    space_x = cx - (bmp->w/2 * line_dx) - (bmp->h/2 * line_dy);
    space_y = cy - (bmp->w/2 * line_dy) + (bmp->h/2 * line_dx);
    
    // make a pointer to the tiles for the current frame
        
    if (is_memory_bitmap (bmp))
    {
        switch (bitmap_color_depth(bmp))
        {
            case 8: ppfunc = memory_putpixel; 
                    gpfunc = memory_getpixel; break;
            case 15:
            case 16:ppfunc = memory_putpixel_15_or_16_bpp; 
                    gpfunc = memory_getpixel_15_or_16_bpp; break;
            case 24:
            case 32:ppfunc = memory_putpixel_32; 
                    gpfunc = memory_getpixel_32; break;
        }
    }
    else 
    {
        ppfunc = putpixel;
        gpfunc = getpixel;
    }
    
    for (screen_y = 0; screen_y < bmp->h; screen_y++)
    {
        old_space_x = space_x;
        old_space_y = space_y;
        
        // go through all points in this screen line
        for (screen_x = 0; screen_x < bmp->w; screen_x++)
        {
            int index, destx, desty, srcx, srcy, pixel;
            int mx, my;
            srcx = fixtoi (space_x);
            srcy = fixtoi (space_y);
            mx = srcx >> tilew_log;
            my = srcy >> tileh_log;
            
            if (mx >= 0 && my >= 0 && mx < map->w && my < map->h)
            { 
                index = teg_mapget
                    (map, 0, (srcx >> tilew_log) & map_mask_x,
                          (srcy >> tileh_log) & map_mask_y);
                
                if (index >= 0 && index < map->tilelist->tilenum)
                {
                    index = frametiles[index].index;
                    destx = (srcx & tile_mask_x)
                        + (index % tiles_per_row) * tiles->tilew;
                    desty = (srcy & tile_mask_y)
                        + (index / tiles_per_row) * tiles->tileh;
        
                    pixel = getpixel (tile, destx, desty);
                    if (pixel != mask_color)
                    {
                        ppfunc (bmp, screen_x, screen_y, pixel);                        
                    }
                                       
                }
            }
            space_x += line_dx;
            space_y += line_dy;
        }        
        space_x = old_space_x + line_dy;
        space_y = old_space_y - line_dx;
    }
}

void test_rot ()
{
    BITMAP *sprite, *buffer;
    int quit = FALSE;
    fixed angle = itofix (0);
    fixed x = 0, y = 0;
    fixed dx = 0, dy = 0;
    fixed speed = 0;
    int i, j, r2;
    int frame = 0;

    // to avoid flicker the program makes use of a double buffering system
    buffer = create_bitmap (screen->w, screen->h);
    // create a 64x64 tile bitmap and draw something on it.

    // create another bitmap and draw something to it
    // this bitmap contains the object
    sprite = create_bitmap (64, 64);
    clear_to_color (sprite, makecol (255, 0, 255));
    for (i = 0; i < 64; i++)
    {
        for (j = 0; j < 64; j++)
        {
            r2 = (32 - i) * (32 - i) + (32 - j) * (32 - j);
            if (r2 < 30 * 30)
            {
                int color;
                r2 = (24 - i) * (24 - i) + (24 - j) * (24 - j);
                color = 255 - (4 * fixtoi(fsqrt(itofix(r2))));
                putpixel (sprite, i, j, makecol (color, color, color));
            }
        }
    }

    while (!quit)
    {
        // act on keyboard input
        if (key[KEY_ESC]) quit = TRUE;
        if (key[KEY_UP] && speed < itofix (5))
            speed += ftofix (0.1);
        if (key[KEY_DOWN] && speed > itofix (-5))
            speed -= ftofix (0.1);
        if (key[KEY_LEFT])
            angle = (angle - itofix (3)) & 0xFFFFFF;
        if (key[KEY_RIGHT])
            angle = (angle + itofix (3)) & 0xFFFFFF;
        dx = fmul (speed, fcos (angle));
        dy = fmul (speed, fsin (angle));

        x += dx;
        y += dy;

        clear_bitmap (buffer);
        teg_masked_rot_map (buffer, map, angle, x, y, frame);
        draw_sprite (buffer, sprite, (buffer->w - sprite->w) / 2, (buffer->h - sprite->h) / 2);
        frame = (-frame) + 1;
        vsync();
        blit (buffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);

    }
    destroy_bitmap (sprite);
    destroy_bitmap (buffer);
}


int main ()
{
    // initialise allegro
    if (allegro_init () < 0)
    {
        allegro_message ("Error: Could not initialise allegro");
        return -1;
    }
    // initialise gfx mode
    set_color_depth(16);
    if (set_gfx_mode (GFX_AUTODETECT_WINDOWED, 640, 480, 0, 0) < 0)
    {
        allegro_message ("Error: Could not set graphics mode");
        return -1;
    }
    // initialise keyboard
    install_keyboard ();
    clear_keybuf ();
	teg_install();
	
    data = teg_load_datafile ("../data//rot.dat");
    tilelist = find_datafile_object (data, "ROT")->dat;
    map = teg_loadmap ("../data//rot1.map", tilelist);

    // call the example function
    test_rot ();

	teg_uninstall();
    // exit allegro
    allegro_exit ();

    return 0;

} END_OF_MAIN ();

