//     :     :     :     :     :     :     :     :     :     :
//   --------:  ---------:  ---------:  ---------:  ----     :
// ..|      |*..|       |*..|       |*..|       |*..|  |*....:.
//   |      |*  |       |*  |       |*  |       |*  |  |*    :
//   ---  ---*  |  |-----*  |  |-----*  |  |-----*  |  |*    :
// ....|  |*.:..|  |--****..|  |-----*..|  |--****..|  |*....:.
//     |  |* :  |    |*  :  |  |--  |*  |    |*  :  |  |*    :
//     |  |* :  |  |--*  :  |  |*|  |*  |  |--*  :  |  |*    :
// ....|  |*.:..|  |-----:..|  ---  |*..|  |-----:..|  -----.:.
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
// ....----*.:..---------*..---------*..---------*..--------*:.
//     :**** :   *********   *********   *********   ********:
//     :     :     :     :     :     :     :     :     :     :
//
// Version 0.81
// Copyright (c) 2002-2007
// Martijn "Amarillion" van Iersel.
//
// This file is part of the Tegel tilemapping add-on for allegro
//
//    Tegel is free software; you can redistribute it and/or modify
//    it under the terms of the GNU Lesser General 
//    Public License as published by the Free Software 
//    Foundation; either version 2.1 of the License, or
//    (at your option) any later version.
//
//    Tegel is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public License
//    along with Tegel; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
// official website:
// http://tegel.sourceforge.net/
//
// I hope you like this program. Please send 
// questions, comments, flames and feature 
// requests to amarillion@yahoo.com.
//
#include "tegel.h"
#include <assert.h>
#include <string.h>

void usage_message()
{
	allegro_message (
"Usage:\n\
	extiles datafile.dat [tilelist.tll]\n\
	if you do not provide a tilelist file, the datafile should contain a tilelist object.\n\
");
}

void run(int argc, char **argv)
{
    DATAFILE *data = NULL;
	DATAFILE *obj = NULL;
    TEG_TILELIST *tilelist = NULL;
	TEG_TILELIST *loaded_tilelist = NULL;
    int i;
	int error = FALSE;
	
    teg_install();
    
	// check arguments list for a filename ending in .dat and load it.
	
	for (i = 1; i < argc && data == NULL; ++i)
	{
		if (strcmp (get_extension (argv[i]), "dat") == 0)
		{
			data = teg_load_datafile (argv[i]);
			if (!data) 
			{
				error = TRUE;
				allegro_message (teg_error);
			}
		}
	}
	
	if (!data && !error)
	{
		error = TRUE;
		usage_message();
	}
	
	// check arguments list for a filename ending in .tll and load it.
	if (!error) for (i = 1; i < argc && loaded_tilelist == NULL; ++i)
	{
		if (strcmp (get_extension (argv[i]), "tll") == 0)
		{
			loaded_tilelist = teg_loadtiles (argv[i], data);
    		if (!loaded_tilelist) 
			{
				error = TRUE;
				allegro_message (teg_error);
			}
			tilelist = loaded_tilelist;	
		}
	}
	
	if (!error && !tilelist)
	{
		for (obj = data; obj->type != DAT_END && obj->type != DAT_TEG_TILELIST; ++obj);
		if (obj->type == DAT_END)
		{
			error = TRUE;
			usage_message();
		}
		else
		{
			tilelist = (TEG_TILELIST*)obj->dat;
		}
	}
	
	if (!error)
	{		
		int x = 0, y = 0;		
		int columnwidth = 24 + 32 + tilelist->animsteps * (4 + tilelist->tilew);
		int rowheight = TEG_MAX (text_height (font), tilelist->tileh) + 4;
		
		int i, j;
		for (i = 0; i < tilelist->tilenum && x < SCREEN_W; ++i)
		{
			textprintf_ex (screen, font, x, y, GREEN, -1, "%i", i);
			textprintf_ex (screen, font, x + 24, y, YELLOW, -1, "%i", tilelist->tiles[i].flags);
			for (j = 0; j < tilelist->animsteps; ++j)
			{
				teg_drawtile (screen, tilelist, 
					i + j * tilelist->tilenum, 
				x + 24 + 32 + j * (tilelist->tilew + 4), y);
			}
			y += rowheight;
			if (y > SCREEN_H - rowheight)
			{
				x += columnwidth;
				y = 0;
			}
		}	
		readkey();
	}
	
    // teg_destroytiles is going to be obsolete
    if (loaded_tilelist) teg_destroytiles (loaded_tilelist);    
    teg_unload_datafile (data);
        
    if (!error) readkey();        
    teg_uninstall();
}


int main (int argc, char **argv)
{
    // initialise allegro
    if (allegro_init () < 0)
    {
        allegro_message ("Error: Could not initialise allegro");
        return -1;
    }
    // initialise gfx mode
    if (set_gfx_mode (GFX_AUTODETECT, 800, 600, 0, 0) < 0)
    {
        allegro_message ("Error: Could not set graphics mode");
        return -1;
    }
    // initialise keyboard
    install_keyboard ();
    clear_keybuf ();

    // call the example function
    run (argc, argv);

    // exit allegro
    allegro_exit ();

    return 0;

} END_OF_MAIN ();

