//     :     :     :     :     :     :     :     :     :     :
//   --------:  ---------:  ---------:  ---------:  ----     :
// ..|      |*..|       |*..|       |*..|       |*..|  |*....:.
//   |      |*  |       |*  |       |*  |       |*  |  |*    :
//   ---  ---*  |  |-----*  |  |-----*  |  |-----*  |  |*    :
// ....|  |*.:..|  |--****..|  |-----*..|  |--****..|  |*....:.
//     |  |* :  |    |*  :  |  |--  |*  |    |*  :  |  |*    :
//     |  |* :  |  |--*  :  |  |*|  |*  |  |--*  :  |  |*    :
// ....|  |*.:..|  |-----:..|  ---  |*..|  |-----:..|  -----.:.
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
// ....----*.:..---------*..---------*..---------*..--------*:.
//     :**** :   *********   *********   *********   ********:
//     :     :     :     :     :     :     :     :     :     :
//
// Version 0.81
// Copyright (c) 2002-2007
// Martijn "Amarillion" van Iersel.
//
// This file is part of the Tegel tilemapping add-on for allegro
//
//    Tegel is free software; you can redistribute it and/or modify
//    it under the terms of the GNU Lesser General 
//    Public License as published by the Free Software 
//    Foundation; either version 2.1 of the License, or
//    (at your option) any later version.
//
//    Tegel is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public License
//    along with Tegel; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
// official website:
// http://tegel.sourceforge.net/
//
// I hope you like this program. Please send 
// questions, comments, flames and feature 
// requests to amarillion@yahoo.com.
//
#include "tegel.h"
#include <assert.h>
#include <stdio.h>

#define TEG_MIN(x,y)     (((x) < (y)) ? (x) : (y))

TEG_MAP *teg_create_copy (TEG_MAP *src)
{
    TEG_MAP* result = teg_createmap (src->dl, src->w, src->h, src->tilelist);
    teg_mapcopy (src, result,
       0,0,0, 0,0,0,
        src->dl,
        src->w,
        src->h);
    return result;
}

/*
resize a map. new parts are emptied, old parts retained as good
as possible.
*/
void teg_resizemap (TEG_MAP **map, int newdl, int neww, int newh)
{
    TEG_MAP* newmap = teg_createmap (newdl, neww, newh, (*map)->tilelist);
    if (newmap != NULL)
    {
        teg_mapcopy ((*map), newmap, 0, 0, 0, 0, 0, 0,
            TEG_MIN (newdl, (*map)->dl),
            TEG_MIN (neww, (*map)->w),
            TEG_MIN (newh, (*map)->h));
        teg_destroymap (*map);
        (*map) = newmap;
    }
}

void teg_maprect (TEG_MAP *destmap,
    int destl, int destx, int desty,
    int dl, int w, int h, int value)
{
    int l, x, y;
    for (l = 0; l < dl; l++)
        for (x = 0; x < w; x++)
            for (y = 0; y < h; y++)
                if (destl + l < destmap->dl &&
                    destx + x < destmap->w &&
                    desty + y < destmap->h &&
                    destl + l >= 0 &&
                    destx + x >= 0 &&
                    desty + y >= 0)
    {
        teg_mapput (destmap, destl + l, destx + x, desty + y, value);
    }
}

/*
generic map copy function. src and dest may be the same.
part that handles overlapping regions not implemented yet.
*/
void teg_mapcopy (TEG_MAP *srcmap, TEG_MAP* destmap,
    int srcl, int srcx, int srcy,
    int destl, int destx, int desty,
    int dl, int w, int h)
{
    int l, x, y;
    for (l = 0; l < dl; l++)
        for (x = 0; x < w; x++)
            for (y = 0; y < h; y++)
                if (
                    destl + l < destmap->dl &&
                    destx + x < destmap->w &&
                    desty + y < destmap->h &&
                    destl + l >= 0 &&
                    destx + x >= 0 &&
                    desty + y >= 0 &&
                    srcl + l < srcmap->dl &&
                    srcx + x < srcmap->w &&
                    srcy + y < srcmap->h &&
                    srcl + l >= 0 &&
                    srcx + x >= 0 &&
                    srcy + y >= 0
                    )
                    teg_mapput (destmap,
                        destl + l, destx + x, desty + y,
                        teg_mapget (srcmap,
                            srcl + l, srcx + x, srcy + y));
}

// save a tilemap
void teg_savemap (TEG_MAP* map, const char *filename)
{
    FILE *fout;
    fout = fopen (filename, "wb");
    if (fout != NULL)
    {
        int temp = 1;
        fwrite ("MIMP", 4, 1, fout);
        fwrite (&temp, 4, 1, fout);
        fwrite (&map->w, 4, 1, fout);
        fwrite (&map->h, 4, 1, fout);
        fwrite (&map->dl, 4, 1, fout);
        fwrite (map->data, map->w * map->h * map->dl * 4, 1, fout);
    }
    fclose (fout);
}

void teg_drawtile (BITMAP *dest, TEG_TILELIST *tiles, int index, int x, int y)
{
    assert (tiles != NULL);
    assert (dest != NULL);
    assert (index >= 0);
    assert (index < (tiles->tilenum * tiles->animsteps));
    switch (tiles->flags & 3)
    {
        case TEG_TILEFLAG_BMP:
            draw_sprite (dest, tiles->tiles[index].bmp, x, y);
            break;
        case TEG_TILEFLAG_RLE:
            draw_rle_sprite (dest, tiles->tiles[index].rle, x, y);
            break;
        case TEG_TILEFLAG_LARGE:
            {
            int tiles_per_row = tiles->largebmp->w / tiles->tilew;
            assert (tiles->largebmp != NULL);
            blit (tiles->largebmp, dest,
                (index % tiles_per_row) * tiles->tilew,
                (index / tiles_per_row) * tiles->tileh,
                x, y,
                tiles->tilew, tiles->tileh);
            }
            break;
        case TEG_TILEFLAG_LARGEI:
            {
            int tiles_per_row = tiles->largebmp->w / tiles->tilew;
            assert (tiles->largebmp != NULL);
            index = tiles->tiles[index].index;            
            blit (tiles->largebmp, dest,
                (index % tiles_per_row) * tiles->tilew,
                (index / tiles_per_row) * tiles->tileh,
                x, y,
                tiles->tilew, tiles->tileh);
            }
            break;
            break;
    }
}
