//     :     :     :     :     :     :     :     :     :     :
//   --------:  ---------:  ---------:  ---------:  ----     :
// ..|      |*..|       |*..|       |*..|       |*..|  |*....:.
//   |      |*  |       |*  |       |*  |       |*  |  |*    :
//   ---  ---*  |  |-----*  |  |-----*  |  |-----*  |  |*    :
// ....|  |*.:..|  |--****..|  |-----*..|  |--****..|  |*....:.
//     |  |* :  |    |*  :  |  |--  |*  |    |*  :  |  |*    :
//     |  |* :  |  |--*  :  |  |*|  |*  |  |--*  :  |  |*    :
// ....|  |*.:..|  |-----:..|  ---  |*..|  |-----:..|  -----.:.
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
// ....----*.:..---------*..---------*..---------*..--------*:.
//     :**** :   *********   *********   *********   ********:
//     :     :     :     :     :     :     :     :     :     :
//
// Version 0.81
// Copyright (c) 2002-2007
// Martijn "Amarillion" van Iersel.
//
// This file is part of the Tegel tilemapping add-on for allegro
//
//    Tegel is free software; you can redistribute it and/or modify
//    it under the terms of the GNU Lesser General 
//    Public License as published by the Free Software 
//    Foundation; either version 2.1 of the License, or
//    (at your option) any later version.
//
//    Tegel is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public License
//    along with Tegel; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
// official website:
// http://tegel.sourceforge.net/
//
// I hope you like this program. Please send 
// questions, comments, flames and feature 
// requests to amarillion@yahoo.com.
//
#include <stdlib.h>
#include <string.h>
#include "tegfile.h"
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <assert.h>

file_driver_t file_driver;

file_driver_t file_set_driver(file_driver_t driver)
{
	file_driver_t cur_file_driver = file_driver;

	file_driver = driver;

	return cur_file_driver;
}

file_t file_open(const char *name, int flags)
{
	file_t file = NULL;

	assert(name);

	file = calloc(1, sizeof(struct file_s));
	assert (file);
        
	file->driver = file_driver;

	file->data = file->driver.open(name, flags);
	if(!file->data) {
		free(file);
		return NULL;
	}

	return file;
}

void file_close(file_t file)
{
	assert(file);

	file->driver.close(file->data);

	free(file);
}

size_t file_read(file_t file, void *data, size_t size)
{
	assert(file);
	assert(data);

	return file->driver.read(file->data, data, size);
}

size_t file_write(file_t file, void *data, size_t size)
{
	assert(file);
	assert(data);

	return file->driver.write(file->data, data, size);
}

off_t file_seek(file_t file, off_t off, int whence)
{
	assert(file);

	return file->driver.seekf(file->data, off, whence);
}

off_t file_tell(file_t file)
{
	assert(file);

	return file->driver.tell(file->data);
}

char *file_error(void)
{
	return file_driver.err();
}

char *file_gets(file_t file, char *buf, size_t size)
{
	unsigned int i;
	assert(file);
	assert(buf);

	/*
	//Older implementation, doesn't seem to work properly with eof.
	for(i=0; i<size-1; i++) {
		int ret = file_read(file, &buf[i], sizeof(char));
		if(ret == -1)
			return NULL;
		if(ret == 0)
			break;
		if(buf[i] == '\n')
			break;
	}
	*/
	for(i=0; i<size-1; i++) {
		int ret = file_read(file, &buf[i], sizeof(char));
		if(ret <= 0)
		{
			buf[i] = 0;
			return NULL;
		}
		if(buf[i] == '\n')
			break;
	}
	buf[++i] = 0;
	return buf;
}

char *file_gets_ex(file_t file, char *buf, size_t size)
{
	unsigned int i;
	assert(file);
	assert(buf);

	for(i=0; i<size-1; i++) {
		int ret = file_read(file, &buf[i], sizeof(char));
		if(ret <= 0)
		{
			buf[i] = 0;
			return NULL;
		}
		if(buf[i] == '\n')
			break;
		if(buf[i] == '\r')
		{
			// look ahead, and if a \n comes next, skip it
			// otherwise move back one char in the stream.
			// but break anyway
			if (i < size-1)
			{
				char c;
				int ret2 = file_read (file, &c, sizeof(char));
				if (ret2 > 0 && c != '\n')
				{
					file_seek (file, -1, SEEK_CUR);
				}					
			}
			buf[i] = '\n';
			break;
		}			
	}
	buf[++i] = 0;

	return buf;
}

int file_puts(file_t file, char *buf)
{
	int len = 0;
	assert(file);
	assert(buf);

	len = strlen(buf);

	return file_write(file, buf, len);
}

int file_getc(file_t file)
{
	assert(file);

	return file->driver.fgetc(file->data);
}

int file_putc(file_t file, int ch)
{
	assert(file);

	return file->driver.fputc(file->data, ch);
}

int file_putw(file_t file, int w)
{
	int b1, b2, b3, b4;

	b1 = (int)((w & 0xFF000000L) >> 24);
   b2 = (int)((w & 0x00FF0000L) >> 16);
   b3 = (int)((w & 0x0000FF00L) >> 8);
   b4 = (int)w & 0x00FF;

	if((int)file_putc(file, b4) == b1)
		if((int)file_putc(file, b3) == b2)
			if((int)file_putc(file, b2) == b3)
				if((int)file_putc(file, b1) == b4)
					return w;

	return EOF;
}

int file_getw(file_t file)
{
	int w = 0;
	int c1=0, c2=0, c3=0, c4=0;

	c1 = file_getc(file) << 24;
	c2 = file_getc(file) << 16;
	c3 = file_getc(file) << 8;
	c4 = file_getc(file);

	w = (c4 | c3 | c2 | c1);

	return w;
}

long file_putuw(file_t f, long l)
{
   int b1, b2, b3, b4;

   b1 = (int)((l & 0xFF000000L) >> 24);
   b2 = (int)((l & 0x00FF0000L) >> 16);
   b3 = (int)((l & 0x0000FF00L) >> 8);
   b4 = (int)l & 0x00FF;

   if (file_putc(f,b1)==b1)
      if (file_putc(f,b2)==b2)
	 if (file_putc(f,b3)==b3)
	    if (file_putc(f,b4)==b4)
	       return l;

   return EOF;
}

long file_getuw(file_t f)
{
   int b1, b2, b3, b4;

   if ((b1 = file_getc(f)) != EOF)
      if ((b2 = file_getc(f)) != EOF)
	 if ((b3 = file_getc(f)) != EOF)
	    if ((b4 = file_getc(f)) != EOF)
	       return (((long)b1 << 24) | ((long)b2 << 16) |
		       ((long)b3 << 8) | (long)b4);

   return EOF;
}


static char *stdiofile_error_str = NULL;

static void stdiofile_set_error(char *str)
{
	int len = strlen(str);
	stdiofile_error_str = realloc(stdiofile_error_str, len+1);
	assert (stdiofile_error_str);

        if (stdiofile_error_str)
        {
            memset(stdiofile_error_str, 0, len+1);
            strncpy(stdiofile_error_str, str, len);
        }
}

file_data_t stdiofile_open(const char *name, int flags)
{
	char mode[] = { 0, 0, 0, 0, 0 };
	FILE *f;
	
	if(flags & FFLAG_READ && flags & FFLAG_WRITE) {
		if(flags & FFLAG_TRUNC)
			strncpy(mode, "w+b", 3);
		else
			strncpy(mode, "r+b", 3);
	} else
	if(flags & FFLAG_READ) {
		strncpy(mode, "rb", 2);
	} else	
	if(flags & FFLAG_WRITE) {
		strncpy(mode, "wb", 2);
	}
	
	
	f = fopen(name, mode);
	if(!f) 
		stdiofile_set_error(strerror(errno));
	
	return f;
}

void stdiofile_close(file_data_t data)
{
	FILE *f = (FILE *)data;
	assert(f);
	
	
	if(fclose(f) == EOF)
		stdiofile_set_error(strerror(errno));
}

int stdiofile_putc(file_data_t data, int c)
{
	FILE *f = (FILE *)data;
	int ret = 0;
	
	ret = fputc(c, f);
	if(ret == EOF)
		stdiofile_set_error(strerror(errno));
	
	return ret;
}

int stdiofile_getc(file_data_t data)
{
	FILE *f = (FILE *)data;
	int ret = 0;
	
	ret = fgetc(f);
	if(ret == EOF)
		stdiofile_set_error(strerror(errno));
	
	return ret;
}

size_t stdiofile_read(file_data_t data, void *buf, size_t size)
{
	FILE *f = (FILE *)data;
	size_t sz = 0;
	assert(f);
	assert(buf);
	
	sz = fread(buf, 1, size, f);
	if(sz != size)
		stdiofile_set_error(strerror(errno));
	
	return sz;
}

size_t stdiofile_write(file_data_t data, void *buf, size_t size)
{
	FILE *f = (FILE *)data;
	size_t sz = 0;
	assert(f);
	assert(buf);
	
	sz = fwrite(buf, 1, size, f);
	if(sz != size)
		stdiofile_set_error(strerror(errno));
	
	return sz;
}

off_t stdiofile_seekf(file_data_t data, off_t off, int whence)
{
	FILE *f = (FILE *)data;
	off_t oft = 0;
	assert(f);
	assert(whence == SEEK_SET || whence == SEEK_CUR || whence == SEEK_END);
	
	
	oft = fseek(f, off, whence);
	if(oft < 0) {
		stdiofile_set_error(strerror(errno));
	}
	return oft;
}

off_t stdiofile_tell(file_data_t data)
{
	FILE *f = (FILE *)data;
	off_t oft = 0;
	
	oft = ftell(f);
	if(oft < 0) {
		stdiofile_set_error(strerror(errno));
	}
	
	return oft;
}

char *stdiofile_error(void)
{
	return stdiofile_error_str;
}

file_driver_t stdiofile_driver = {
	stdiofile_open,
	stdiofile_close,
	stdiofile_putc,
	stdiofile_getc,
	stdiofile_read,
	stdiofile_write,
	stdiofile_seekf,
	stdiofile_tell,
	stdiofile_error
};

static char *memfile_error_str = NULL;

typedef struct memfile_data_s {
    size_t size;
    char *buffer;
    int pos;
} memfile_data_t;

static void memfile_set_error(char *str)
{
	int len = strlen(str);
	memfile_error_str = realloc(memfile_error_str, len+1);
	assert(memfile_error_str);
	if (memfile_error_str)
        {
            memset(memfile_error_str, 0, len+1);
            strncpy(memfile_error_str, str, len);
        }
}

file_t memfile_openbuf(char *buffer, size_t size)
{
    file_t f;
    memfile_data_t *d;    
    d = malloc (sizeof (memfile_data_t));
    f = calloc(1, sizeof(struct file_s));
    assert (d);
    assert (f);
    d->size = size;
    d->buffer = buffer;
    d->pos = 0;
    f->driver = memfile_driver;
    f->data = d;
    f->flags = 0;
    return f;    
}

file_data_t memfile_open(const char *name, int flags)
{
    return NULL;
}

void memfile_close(file_data_t data)
{
    memfile_data_t *d = (memfile_data_t *)data;
    free (d);
}

int memfile_putc(file_data_t data, int c)
{
    return 0;
}

int memfile_getc(file_data_t data)
{
    memfile_data_t *d = (memfile_data_t *)data;
    
    int ret = 0;
    ret = d->buffer[d->pos];
    ++(d->pos);
    if (d->pos >= d->size)
    {
        ret = EOF;        
        memfile_set_error ("End of file");
    }

    return ret;
}

size_t memfile_read(file_data_t data, void *buf, size_t size)
{
    // how many can I read?
    int toread = size;
    memfile_data_t *d = (memfile_data_t *)data;
    
    if (d->pos + toread >= d->size)
    {
        toread = (d->size - d->pos);
    }
    memcpy (buf, d->buffer + d->pos, toread);
    d->pos += toread;
    return toread;
}

size_t memfile_write(file_data_t data, void *buf, size_t size)
{
	memfile_set_error ("memfile_write not implemented");
    return -1;
}

off_t memfile_seekf(file_data_t data, off_t off, int whence)
{
	memfile_data_t *d = (memfile_data_t *)data;
	
	off_t newpos = 0;
	
	switch (whence)
	{
		case SEEK_SET:
			newpos = off;
			break;
		case SEEK_CUR:
			newpos += d->pos + off;
			break;
		case SEEK_END:
			newpos = d->size + off;
			break;
		default:
			memfile_set_error ("Incorrect whence argument for memfile_seekf, must be SEEK_SET, SEEK_CUR or SEEK_END");
			return -1;
	}

	if (newpos >= 0 && newpos < d->size)
	{
		d->pos = newpos;
		return 0;
	}
	else
	{
		memfile_set_error ("memfile_seekf Out of bounds");
		return -1;
	}
}

off_t memfile_tell(file_data_t data)
{
    memfile_data_t *d = (memfile_data_t *)data;
	return d->pos;
}

char *memfile_error(void)
{
    return memfile_error_str;
}

file_driver_t memfile_driver = {
	memfile_open,
	memfile_close,
	memfile_putc,
	memfile_getc,
	memfile_read,
	memfile_write,
	memfile_seekf,
	memfile_tell,
	memfile_error
};
