//     :     :     :     :     :     :     :     :     :     :
//   --------:  ---------:  ---------:  ---------:  ----     :
// ..|      |*..|       |*..|       |*..|       |*..|  |*....:.
//   |      |*  |       |*  |       |*  |       |*  |  |*    :
//   ---  ---*  |  |-----*  |  |-----*  |  |-----*  |  |*    :
// ....|  |*.:..|  |--****..|  |-----*..|  |--****..|  |*....:.
//     |  |* :  |    |*  :  |  |--  |*  |    |*  :  |  |*    :
//     |  |* :  |  |--*  :  |  |*|  |*  |  |--*  :  |  |*    :
// ....|  |*.:..|  |-----:..|  ---  |*..|  |-----:..|  -----.:.
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
// ....----*.:..---------*..---------*..---------*..--------*:.
//     :**** :   *********   *********   *********   ********:
//     :     :     :     :     :     :     :     :     :     :
//
// Version 0.81
// Copyright (c) 2002-2007
// Martijn "Amarillion" van Iersel.
//
// This file is part of the Tegel tilemapping add-on for allegro
//
//    Tegel is free software; you can redistribute it and/or modify
//    it under the terms of the GNU Lesser General 
//    Public License as published by the Free Software 
//    Foundation; either version 2.1 of the License, or
//    (at your option) any later version.
//
//    Tegel is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public License
//    along with Tegel; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
// official website:
// http://tegel.sourceforge.net/
//
// I hope you like this program. Please send 
// questions, comments, flames and feature 
// requests to amarillion@yahoo.com.
//
#ifndef __FILE_H__
#define __FILE_H__

#define __USE_FILE_OFFSET64 1
#define __USE_LARGEFILE64 1

#include <sys/types.h>

#define FFLAG_READ 1
#define FFLAG_WRITE 2

/* following flags not supported by gzfile driver */
#define FFLAG_RW (FFLAG_READ | FFLAG_WRITE)
#define FFLAG_APPEND 4
#define FFLAG_RA (FFLAG_READ | FFLAG_APPEND)
#define FFLAG_TRUNC 8

typedef void *file_data_t;
typedef file_data_t (*file_open_func_t)(const char *, int);
typedef void (*file_close_func_t)(file_data_t);
typedef int (*file_putc_func_t)(file_data_t, int);
typedef int (*file_getc_func_t)(file_data_t);
typedef size_t (*file_read_func_t)(file_data_t, void *, size_t);
typedef size_t (*file_write_func_t)(file_data_t, void *, size_t);
typedef off_t (*file_seek_func_t)(file_data_t, off_t, int);
typedef off_t (*file_tell_func_t)(file_data_t);
typedef char *(*file_error_func_t)(void);

typedef struct file_driver_s {
	file_open_func_t open;
	file_close_func_t close;
	file_putc_func_t fputc;
	file_getc_func_t fgetc;
	file_read_func_t read;
	file_write_func_t write;
	file_seek_func_t seekf;
	file_tell_func_t tell;
	file_error_func_t err;
} file_driver_t;

typedef struct file_chunk_s {
	off_t start;
	size_t length;
	file_data_t data;
} file_chunk_t;

typedef struct file_s {
	int flags;
	file_data_t data;
	file_driver_t driver;
} *file_t;

file_driver_t file_set_driver(file_driver_t driver);
file_t file_open(const char *name, int flags);
void file_close(file_t file);
size_t file_read(file_t, void *, size_t);
size_t file_write(file_t, void *, size_t);
off_t file_seek(file_t, off_t, int);
off_t file_tell(file_t);
char *file_error(void);

// cross platform version of file_gets, reads until either \n, \r or \r\n occurs.
char *file_gets_ex(file_t file, char *buf, size_t size);

char *file_gets(file_t, char *, size_t);
int file_puts(file_t, char *);
int file_getc(file_t);
int file_putc(file_t, int);

/* reads/writes a 32bit integer in an endian independent format */
int file_putw(file_t, int);
int file_getw(file_t);

long file_putuw(file_t f, long l);
long file_getuw(file_t f);

/* file drivers... */

extern file_driver_t stdiofile_driver;
extern file_driver_t memfile_driver;

file_t memfile_openbuf(char *buffer, size_t size);

#endif /* __FILE_H__ */
