//     :     :     :     :     :     :     :     :     :     :
//   --------:  ---------:  ---------:  ---------:  ----     :
// ..|      |*..|       |*..|       |*..|       |*..|  |*....:.
//   |      |*  |       |*  |       |*  |       |*  |  |*    :
//   ---  ---*  |  |-----*  |  |-----*  |  |-----*  |  |*    :
// ....|  |*.:..|  |--****..|  |-----*..|  |--****..|  |*....:.
//     |  |* :  |    |*  :  |  |--  |*  |    |*  :  |  |*    :
//     |  |* :  |  |--*  :  |  |*|  |*  |  |--*  :  |  |*    :
// ....|  |*.:..|  |-----:..|  ---  |*..|  |-----:..|  -----.:.
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
// ....----*.:..---------*..---------*..---------*..--------*:.
//     :**** :   *********   *********   *********   ********:
//     :     :     :     :     :     :     :     :     :     :
//
// Version 0.81
// Copyright (c) 2002-2007
// Martijn "Amarillion" van Iersel.
//
// This file is part of the Tegel tilemapping add-on for allegro
//
//    Tegel is free software; you can redistribute it and/or modify
//    it under the terms of the GNU Lesser General 
//    Public License as published by the Free Software 
//    Foundation; either version 2.1 of the License, or
//    (at your option) any later version.
//
//    Tegel is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public License
//    along with Tegel; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
// official website:
// http://tegel.sourceforge.net/
//
// I hope you like this program. Please send 
// questions, comments, flames and feature 
// requests to amarillion@yahoo.com.
//
#include <stdio.h>
#include "tegel.h"
#include <assert.h>
#include <memory.h>

// adjust dialog to resolution
// assumes first item in dialog array should be maximized to full screen,
// with the same empty area left as right and the same on top as on bottom
void resize_dialog (DIALOG *d)
{
    DIALOG *item = d;
    int w, h;
    w = item->w + 2 * item->x;
    h = item->h + 2 * item->y;
    assert (d != NULL);
    assert (w != 0);
    assert (h != 0);
    while (item->proc != NULL)
    {
        item->w = item->w * SCREEN_W / w;
        item->x = item->x * SCREEN_W / w;
        item->h = item->h * SCREEN_H / h;
        item->y = item->y * SCREEN_H / h;
        item++;
    }
}

/*
static int close_dialog(void)
{
    return D_CLOSE;
}
*/

#define TEXTFILE_DIALOG_TEXT 1
#define TEXTFILE_DIALOG_TEXTBOX 2

static DIALOG textfile_dialog[] =
{
   // (dialog proc)     (x)   (y)   (w)   (h)   (fg)  (bg)  (key) (flags)  (d1)  (d2)  (dp)  (dp2) (dp3)
   { d_box_proc,        10,    10,    300,  180,  255,  0,    0,    0,       0,    0,    NULL, NULL, NULL },
   { d_text_proc,       14,    14,    300,   16,  255,  0,    0,    0,       0,    0,    NULL, NULL, NULL },
   { d_textbox_proc,    14,    30,   292,  120,  255,  0,    0,    0,       0,    0,    NULL, NULL, NULL },
   { d_button_proc,     140,  160,   40,   20,  255,  0,   13,    D_EXIT,  0,    0,    "OK", NULL, NULL },
   { NULL,              0,    0,      0,    0,    0,  0,    0,    0,       0,    0,    NULL, NULL, NULL }
};

/*
GUI routine to display the contents of a text file,
could be used to display help info.
*/
void teg_showtext (char *title, char *filename)
{
   int size;
   char *text = NULL;
   FILE *f = NULL;
   int error = FALSE;

   size = file_size(filename);
   if (size <= 0) error = TRUE;
   else
   {
       text = malloc(size+1);
       if (!text)
       {
           error = TRUE;
       } else
       {
         memset (text, 0, size+1);
         f = fopen (filename, "rt");
         fread (text, 1, size, f);
       }
   }
   if (f != NULL) fclose (f);
   if (error)
   {
      alert("Error reading textfile", NULL, NULL, "OK", NULL, 13, 0);
   }
   else
   {
       textfile_dialog[TEXTFILE_DIALOG_TEXT].dp = title;
       textfile_dialog[TEXTFILE_DIALOG_TEXTBOX].dp = text;
       resize_dialog (textfile_dialog);
       set_dialog_color(textfile_dialog, gui_fg_color, gui_bg_color);
       popup_dialog (textfile_dialog, -1);
   }
   if (text != NULL) free (text);
}
