//     :     :     :     :     :     :     :     :     :     :
//   --------:  ---------:  ---------:  ---------:  ----     :
// ..|      |*..|       |*..|       |*..|       |*..|  |*....:.
//   |      |*  |       |*  |       |*  |       |*  |  |*    :
//   ---  ---*  |  |-----*  |  |-----*  |  |-----*  |  |*    :
// ....|  |*.:..|  |--****..|  |-----*..|  |--****..|  |*....:.
//     |  |* :  |    |*  :  |  |--  |*  |    |*  :  |  |*    :
//     |  |* :  |  |--*  :  |  |*|  |*  |  |--*  :  |  |*    :
// ....|  |*.:..|  |-----:..|  ---  |*..|  |-----:..|  -----.:.
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
// ....----*.:..---------*..---------*..---------*..--------*:.
//     :**** :   *********   *********   *********   ********:
//     :     :     :     :     :     :     :     :     :     :
//
// Version 0.81
// Copyright (c) 2002-2007
// Martijn "Amarillion" van Iersel.
//
// This file is part of the Tegel tilemapping add-on for allegro
//
//    Tegel is free software; you can redistribute it and/or modify
//    it under the terms of the GNU Lesser General 
//    Public License as published by the Free Software 
//    Foundation; either version 2.1 of the License, or
//    (at your option) any later version.
//
//    Tegel is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public License
//    along with Tegel; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
// official website:
// http://tegel.sourceforge.net/
//
// I hope you like this program. Please send 
// questions, comments, flames and feature 
// requests to amarillion@yahoo.com.
//
#include "tegmode7.h"

/* draw object just draws a single object at a fixed position, although
this can easily be modified to allow for more objects.
bmp = bitmap to draw to. obj = bitmap for the object
angle, cx, cy define the camera position
*/
void teg_draw_object (BITMAP *bmp, BITMAP *obj,
   fixed object_x, fixed object_y, fixed angle,
   fixed cx, fixed cy, TEG_MODE_7_PARAMS params)
{
    int width, height;
    int screen_y, screen_x;

    // the object in this case is at a fixed coordinate of (160, 100)
    // calculate the position relative to the camera
    fixed obj_x = object_x - cx;
    fixed obj_y = object_y - cy;

    // use a rotation transformation to rotate the object with the camera
    // angle.
    fixed space_x = fmul (obj_x, fcos (angle)) + fmul (obj_y, fsin (angle));
    fixed space_y = -fmul (obj_x, fsin (angle)) + fmul (obj_y, fcos (angle));

    // calculate the screen coordinates that go with that space coordinates
    // by deviding everything by space_x (the distance)
    screen_x = bmp->w/2 + fixtoi (fmul (fdiv (params.scale_x, space_x), space_y));
    screen_y = fixtoi (fdiv (fmul (params.space_z, params.scale_y), space_x)) - params.horizon;

    // the size of the object has to be scaled according to the distance.
    height = fixtoi (obj->h * fdiv(params.obj_scale_y, space_x));
    width = fixtoi (obj->w * fdiv(params.obj_scale_x, space_x));

    // draw the object
    stretch_sprite (bmp, obj, screen_x - width / 2, screen_y - height, width, height);
}

/*
    blits bitmap tile repeating endlessly
*/
void teg_mode_7_sprite (BITMAP *bmp, BITMAP *tile, fixed angle, fixed cx, fixed cy, TEG_MODE_7_PARAMS params)
{
    // current screen position
    int screen_x, screen_y;

    //the distance and horizontal scale of the line we are drawing
    fixed distance, horizontal_scale;

    // masks to make sure we don't read pixels outside the tile
    int mask_x = (tile->w - 1);
    int mask_y = (tile->h -1);

    // step for points in space between two pixels on a horizontal line
    fixed line_dx, line_dy;

    // current space position
    fixed space_x, space_y;

    for (screen_y = 0; screen_y < bmp->h; screen_y++)
    {
        // first calculate the distance of the line we are drawing
        distance = fdiv (fmul (params.space_z, params.scale_y),
            itofix (screen_y + params.horizon));
        // then calculate the horizontal scale, or the distance between
        // space points on this horizontal line
        horizontal_scale = fdiv (distance, params.scale_x);

        // calculate the dx and dy of points in space when we step
        // through all points on this line.
        line_dx = fmul (-fsin(angle), horizontal_scale);
        line_dy = fmul (fcos(angle), horizontal_scale);

        // calculate the starting position
        space_x = cx + fmul (distance, fcos(angle)) - bmp->w/2 * line_dx;
        space_y = cy + fmul (distance, fsin(angle)) - bmp->w/2 * line_dy;

        // go through all points in this screen line
        for (screen_x = 0; screen_x < bmp->w; screen_x++)
        {
            // get a pixel from the tile and put it to screen
            putpixel (bmp, screen_x, screen_y,
                getpixel (tile,
                    fixtoi (space_x) & mask_x,
                    fixtoi (space_y) & mask_y ));
            // go to the next position in space
            space_x += line_dx;
            space_y += line_dy;
        }
    }
}

/*
draw a map in mode7 format.
prerequisites:
   - map is large or largei
   - map->w, map->h, tilew, tileh are powers of 2
*/
void teg_mode_7_map (BITMAP *bmp, TEG_MAP *map, fixed angle, fixed cx, fixed cy, TEG_MODE_7_PARAMS params)
{
    // current screen position
    int screen_x, screen_y;

    //the distance and horizontal scale of the line we are drawing
    fixed distance, horizontal_scale;

    TEG_TILELIST *tiles = map->tilelist;
    BITMAP *tile = map->tilelist->largebmp;

    int tiles_per_row = tile->w / tiles->tilew;

    // masks to make sure we don't read pixels outside the tile
    int tile_mask_x = (tiles->tilew - 1);
    int tile_mask_y = (tiles->tileh -1);
    int map_mask_x = (map->w - 1);
    int map_mask_y = (map->h - 1);
    int tilew_log = 4; // !!!!!!!!!!!!
    int tileh_log = 4;

    // step for points in space between two pixels on a horizontal line
    fixed line_dx, line_dy;

    // current space position
    fixed space_x, space_y;

    for (screen_y = 0; screen_y < bmp->h; screen_y++)
    {
        // first calculate the distance of the line we are drawing
        distance = fdiv (fmul (params.space_z, params.scale_y),
            itofix (screen_y + params.horizon));
        // then calculate the horizontal scale, or the distance between
        // space points on this horizontal line
        horizontal_scale = fdiv (distance, params.scale_x);

        // calculate the dx and dy of points in space when we step
        // through all points on this line.
        line_dx = fmul (-fsin(angle), horizontal_scale);
        line_dy = fmul (fcos(angle), horizontal_scale);

        // calculate the starting position
        space_x = cx + fmul (distance, fcos(angle)) - bmp->w/2 * line_dx;
        space_y = cy + fmul (distance, fsin(angle)) - bmp->w/2 * line_dy;

        // go through all points in this screen line
        for (screen_x = 0; screen_x < bmp->w; screen_x++)
        {
            int index, destx, desty, srcx, srcy;
            srcx = fixtoi (space_x);
            srcy = fixtoi (space_y);
            index = teg_mapget
                (map, 0, (srcx >> tilew_log) & map_mask_x,
                      (srcy >> tileh_log) & map_mask_y);
            //index = 5;
            destx = (srcx & tile_mask_x)
                + (index % tiles_per_row) * tiles->tilew;
            desty = (srcy & tile_mask_y)
                + (index / tiles_per_row) * tiles->tileh;

            // get a pixel from the tile and put it to screen
            putpixel (bmp, screen_x, screen_y,
                getpixel (tile,
                    destx,
                    desty)
            );
            // go to the next position in space
            space_x += line_dx;
            space_y += line_dy;
        }
    }
}
