//     :     :     :     :     :     :     :     :     :     :
//   --------:  ---------:  ---------:  ---------:  ----     :
// ..|      |*..|       |*..|       |*..|       |*..|  |*....:.
//   |      |*  |       |*  |       |*  |       |*  |  |*    :
//   ---  ---*  |  |-----*  |  |-----*  |  |-----*  |  |*    :
// ....|  |*.:..|  |--****..|  |-----*..|  |--****..|  |*....:.
//     |  |* :  |    |*  :  |  |--  |*  |    |*  :  |  |*    :
//     |  |* :  |  |--*  :  |  |*|  |*  |  |--*  :  |  |*    :
// ....|  |*.:..|  |-----:..|  ---  |*..|  |-----:..|  -----.:.
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
// ....----*.:..---------*..---------*..---------*..--------*:.
//     :**** :   *********   *********   *********   ********:
//     :     :     :     :     :     :     :     :     :     :
//
// Version 0.81
// Copyright (c) 2002-2007
// Martijn "Amarillion" van Iersel.
//
// This file is part of the Tegel tilemapping add-on for allegro
//
//    Tegel is free software; you can redistribute it and/or modify
//    it under the terms of the GNU Lesser General 
//    Public License as published by the Free Software 
//    Foundation; either version 2.1 of the License, or
//    (at your option) any later version.
//
//    Tegel is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public License
//    along with Tegel; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
// official website:
// http://tegel.sourceforge.net/
//
// I hope you like this program. Please send 
// questions, comments, flames and feature 
// requests to amarillion@yahoo.com.
//
#include "tegstoks.h"
#include "tilemap.h"
#include <assert.h>
#include <stdio.h>
#include <memory.h>

/*
loading stokseth map from data file to mimap struct

//??? load_stokmap_data (DATAFILE *, TEG_MAP*, TILELIST *, DATAFILE *);

loading stokseth map from plain file to mimap struct

load_stokmap ("filename", DATAFILE *, TEG_MAP**, TILELIST **);
// both map and tilelist are created.
// datafile must be preloaded

saving mimap struct as stokseth plain map

save_stokmap ("filename", TEG_MAP*, //???

*/

// stok is freed
// mi and tiles are created. should be null vars.
void stok2mi (TILEMAP *stok, DATAFILE *data, TEG_MAP **mi, TEG_TILELIST **tiles)
{
    TEG_TILELIST *temptiles = malloc(sizeof(TEG_TILELIST));
    TEG_MAP *tempmap = teg_createmap (stok->layers, stok->xsize, stok->ysize, temptiles);
    int x, y, l, i, t;
    DATAFILE *dataitem;
    int *switcher = malloc (sizeof (int) * 1000);
    memset (switcher, 0, sizeof (int) * 1000);
    // look up tiles in namelist and collect them in the array

    temptiles->tiles = malloc (sizeof(TEG_TILE) * stok->names);
    t = 0;
    assert (temptiles->tiles != NULL);
    for (i = 0; i < stok->names; i++)
    {
        dataitem = find_datafile_object (data, stok->name[i].name);
        assert (stok->name[i].index < 1000);
        temptiles->tiles[t].bmp = NULL;
        temptiles->tiles[t].rle = NULL;
        temptiles->tiles[t].index = 0;
        if (dataitem->type == DAT_RLE_SPRITE &&
            ((RLE_SPRITE*)dataitem->dat)->w == stok->xgrid &&
            ((RLE_SPRITE*)dataitem->dat)->h == stok->ygrid)
        {
           temptiles->tiles[t].rle = (RLE_SPRITE*)dataitem->dat;
           switcher [stok->name[i].index] = t;
           t++;
        }
    }
    temptiles->animsteps = 1;
    temptiles->largebmp = NULL;
    temptiles->flags = TEG_TILEFLAG_RLE;
    temptiles->tilew = stok->xgrid;
    temptiles->tileh = stok->ygrid;
    temptiles->tilenum = t;
    temptiles->bpp = bitmap_color_depth(screen);
    
    for (l = 0; l < stok->layers; l++)
        for (x = 0; x < stok->xsize; x++)
            for (y = 0; y < stok->ysize; y++)
            {
                int j = get_sprite_number (stok, l, x, y);
                int k = 0;
                if (j < 0) k = j; else k = switcher[j];
                teg_mapput (tempmap, l, x, y, k);
            }
    *tiles = temptiles;
    *mi = tempmap;
    free (switcher);
}

void loadstok (char *filename, DATAFILE *data, TEG_MAP **map, TEG_TILELIST **tiles)
{
    TILEMAP *temp = load_tilemap (filename);
    stok2mi (temp, data, map, tiles);
}

// loads mirage file based on preexisting tll file
TEG_MAP *loadpas (char *filename, TEG_TILELIST *tiles)
{
    TEG_MAP *tempmap;
    int i, w, h, layers, objects, bpp, size;
    int l, x, y;
    int temp;
    int *lstart, *lbpp;
    char name[9];
    char *tmpdat;

    FILE *f = fopen (filename, "rb");
    if (f)
    {
        fgetc (f);       // skip one char for unknown reason ? alignment?
        fread (&temp, sizeof(int), 1, f); // ID
        fread (&temp, sizeof(int), 1, f); // version
        fread (name, sizeof(name)-1, 1, f); // version
        name[9] = 0;
        fgetc (f); // skip again
        fread (&w, sizeof(int), 1, f); // w
        fread (&h, sizeof(int), 1, f); // h
        fread (&layers, sizeof(int), 1, f);
        fread (&objects, sizeof(int), 1, f);
        fread (&bpp, sizeof(int), 1, f);
        fread (&size, sizeof(int), 1, f);
        fread (&temp, sizeof(int), 1, f); // unused
        tempmap = teg_createmap (layers, w, h, tiles);

        lstart = malloc (sizeof(int) * layers);
        lbpp = malloc (sizeof(int) * layers);

        for (i = 0; i < layers; i++)
        {
            fread (&temp, sizeof (int), 1, f);
            lstart[i] = temp;
            fread (&temp, sizeof (int), 1, f);
            lbpp[i] = temp;
        }

        tmpdat = malloc (sizeof(char) * bpp * w * h);

        fread (tmpdat, bpp * w * h, 1, f);
        for (l = 0; l < layers; l++)
            for (y = 0; y < h; y++)
                for (x = 0; x < w; x++)
                {
                    // niet helemaal correct,
                    // houdt geen rekening met bpp != 1
                    int index = lstart[l] +lbpp[l] * (x + (y * w));
                    temp = tmpdat[index];
                    teg_mapput (tempmap, l, x, y, temp);
                }
        free (lstart);
        free (lbpp);
        free (tmpdat);

        // skip objects

        fclose (f);
    }
    return tempmap;
}

// loads pas based on preexisting tll file
// in old pascal format (w 2 bytes, h 2 bytes, then data. untested)
TEG_MAP *loadoldpas (char *filename, TEG_TILELIST *tiles)
{
    FILE *f = fopen (filename, "rb");
    int w, h, x, y;
    char *d;
    char buf[4];
    TEG_MAP *tempmap = NULL;
    // read w, h
    fread (buf, 4, 1, f);
    w = buf[0] + buf[1] * 256;
    h = buf[2] + buf[3] * 256;
    // read data
    d = malloc (sizeof(char) * w * h);
    fread (d, w * h, 1, f);
    // convert to map
    tempmap = teg_createmap (0, w, h, tiles);
    for (x = 0; x < w; x ++) for (y = 0; y < h; y++)
    {
        teg_mapput (tempmap, 0, x, y, d[x + w * y]);
    }
    free (d);
    fclose (f);
    return tempmap;
}
