//     :     :     :     :     :     :     :     :     :     :
//   --------:  ---------:  ---------:  ---------:  ----     :
// ..|      |*..|       |*..|       |*..|       |*..|  |*....:.
//   |      |*  |       |*  |       |*  |       |*  |  |*    :
//   ---  ---*  |  |-----*  |  |-----*  |  |-----*  |  |*    :
// ....|  |*.:..|  |--****..|  |-----*..|  |--****..|  |*....:.
//     |  |* :  |    |*  :  |  |--  |*  |    |*  :  |  |*    :
//     |  |* :  |  |--*  :  |  |*|  |*  |  |--*  :  |  |*    :
// ....|  |*.:..|  |-----:..|  ---  |*..|  |-----:..|  -----.:.
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
// ....----*.:..---------*..---------*..---------*..--------*:.
//     :**** :   *********   *********   *********   ********:
//     :     :     :     :     :     :     :     :     :     :
//
// Version 0.81
// Copyright (c) 2002-2007
// Martijn "Amarillion" van Iersel.
//
// This file is part of the Tegel tilemapping add-on for allegro
//
//    Tegel is free software; you can redistribute it and/or modify
//    it under the terms of the GNU Lesser General 
//    Public License as published by the Free Software 
//    Foundation; either version 2.1 of the License, or
//    (at your option) any later version.
//
//    Tegel is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public License
//    along with Tegel; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
// official website:
// http://tegel.sourceforge.net/
//
// I hope you like this program. Please send 
// questions, comments, flames and feature 
// requests to amarillion@yahoo.com.
//
// --------------------------------------------------------------------------
// Level-Editor v1.08 --- Tilemap Library.
// Copyright (c) Henrik Stokseth, 2000.
// modifications by Amarillion

// --------------------------------------------------------------------------
// Include files

#include <string.h>
#include <stdio.h>
#include <allegro.h>

#include "tilemap.h"

//functions below added by Amarillion
/*
  the function find_sprite_in_box searches an area on the tilemap
  for a set of sprites. Which sprites are searched for is given with prop[]
  e.g. if prop[12] = 1 and prop[13] = 0 then the function searches for
  12 but not 13.
  This is usefull for finding out if a certain area contains solid sprites.
  If it does, certain creatures can not go there.
*/
int find_sprite_in_box (TILEMAP *map, int layer, int x, int y, int w, int h, char prop[])
{
    int xco, yco, sprite;
    int found = 0;

    yco = y / map->ygrid;
    while (yco <= ((y + h - 1) / map -> ygrid))
    {
        xco = x / map->xgrid;
        while (xco <= ((x + w - 1) / map -> xgrid))
        {
            sprite = get_sprite_number (map, layer, xco, yco);
            if (sprite >= 0) if (prop[sprite]) found = 1;
            xco++;
        }
        yco++;
    }
    return found;
}

// --------------------------------------------------------------------------
// function called by load_datafile after you have registered it
// with allegro.
void *load_mapdata(PACKFILE *f, long size)
{
    TILEMAP *map;
    int siz;
    int count;

    // load map struct
    siz = sizeof (TILEMAP);
    map = (TILEMAP*)malloc(siz);
    if(!map) return NULL;
    pack_fread (map, siz, f);

    // load data
    siz = tilemap_data_size (map) * sizeof (int);
    map->data = (int*)malloc(siz);
    if(!map->data)
    {
        free (map);
        return NULL;
    }
    pack_fread (map->data, siz, f);

    // load namelist
    if(map->names)
    {
        siz = tilemap_namelist_size (map);
        map->name = (NAMELIST*)malloc(siz);
        if (!map->name)
        {
            free (map->data);
            free (map);
            return NULL;
        }

        for(count=0; count<map->names; count++)
        {
          pack_fread(&map->name[count].index, sizeof(int), f);
          pack_fgets(map->name[count].name, MAX_STRING_LENGTH, f);
          map->name[count].name[strlen(map->name[count].name)] = 0;
        }
    }

    return map;
}

// --------------------------------------------------------------------------
// function called by unload_datafile after you have registered
// it with allegro.
void destroy_mapdata(void *data)
{
    destroy_tilemap ((TILEMAP*)data);
}

// --------------------------------------------------------------------------
// function to register load_mapdata and destroy_mapdata with allegro.
void register_datafile_tilemap ()
{
   register_datafile_object(DAT_MAPDATA, load_mapdata, destroy_mapdata);
}

// --------------------------------------------------------------------------
// calculate data size
int tilemap_data_size (TILEMAP* map)
{
    return map->layers * map->xsize * map->ysize;
}

// --------------------------------------------------------------------------
// calculate namelist size
int tilemap_namelist_size (TILEMAP *map)
{
    return sizeof(NAMELIST) * map->names;
}

// --------------------------------------------------------------------------
// copy a map
TILEMAP *copy_tilemap (TILEMAP* original)
{
    TILEMAP *map;
    int size;

    size = sizeof (TILEMAP);
    map = (TILEMAP*)malloc(size);
    if(!map) return NULL;
    memcpy (map, original, size);
    // copy data
    size = tilemap_data_size (original) * sizeof (int);
    map->data = (int*)malloc(size);
    if(!map->data)
    {
        free (map);
        return NULL;
    }
    memcpy (map->data, original->data, size);

    // copy namelist
    if(map->names)
    {
        size = tilemap_namelist_size (original);
        map->name = (NAMELIST*)malloc(size);
        if (!map->name)
        {
            free (map->data);
            free (map);
            return NULL;
        }
        memcpy (map->name, original->name, size);
    }

    return map;
}

// --------------------------------------------------------------------------
// The get_object_name function


char *get_object_name(DATAFILE *dat, int objnum)
{
  int counter;

  // make sure the object is there
  if(objnum<0) return NULL;
  for(counter=0; dat[counter].type != DAT_END; counter++);
  if(objnum>counter) return NULL;
  
  // return name
  for(counter=0; dat[objnum].prop[counter].type != DAT_END; counter++)
  {
    if(dat[objnum].prop[counter].type == DAT_NAME)
    return dat[objnum].prop[counter].dat;
  }    
  return NULL;
}


// --------------------------------------------------------------------------
// The object_of_type_exists function

int object_of_type_exists(DATAFILE *dat, int type)
{
  int counter;

  for(counter=0; dat[counter].type != DAT_END; counter++)
  {
    if(dat[counter].type == type) return TRUE;
  }
  return FALSE;
}


// --------------------------------------------------------------------------
// The number_of_type function

int number_of_type(DATAFILE *dat, int type)
{
  int counter, counter2 = 0;

  for(counter=0; dat[counter].type != DAT_END; counter++)
  if(dat[counter].type == type) counter2++;
  return counter2;
}


// --------------------------------------------------------------------------
// The get_object_number function

int get_object_number(DATAFILE *dat, int number_of_type, int type)
{
  int counter, counter2 = 0;

  // if found
  for(counter=0; dat[counter].type != DAT_END; counter++)
  {
    if(dat[counter].type == type) counter2++;
    if(number_of_type == counter2) return counter;
  }

  // if not found
  return NO_SPRITE;
}


// --------------------------------------------------------------------------
// The get_max_sprite_width function

int get_max_sprite_width(DATAFILE *dat)
{
  RLE_SPRITE *sprite = NULL;
  int        counter;
  int        tempx = 0;

  for(counter=0; dat[counter].type != DAT_END; counter++)
  {
    if(dat[counter].type == DAT_RLE_SPRITE)
    {
      sprite = dat[counter].dat;
      if(sprite->w > tempx) tempx = sprite->w;
    }
  }
  return tempx;
}


// --------------------------------------------------------------------------
// The get_max_sprite_height function

int get_max_sprite_height(DATAFILE *dat)
{
  RLE_SPRITE *sprite = NULL;
  int        counter;
  int        tempy = 0;

  for(counter=0; dat[counter].type != DAT_END; counter++)
  {
    if(dat[counter].type == DAT_RLE_SPRITE)
    {
      sprite = dat[counter].dat;
      if(sprite->h > tempy) tempy = sprite->h;
    }
  }
  return tempy;
}


// --------------------------------------------------------------------------
// The load_tilemap function
// modified by Amarillion

TILEMAP *load_tilemap(char *filename)
{
  FILE    *mapfile;
  TILEMAP *map;
  int     count;

  mapfile = fopen(filename, "rb");
  if(!mapfile) return NULL;

  map = malloc(sizeof(TILEMAP));
  if(!map) return NULL;
  fread(map, sizeof(TILEMAP), 1, mapfile);

  map->data = malloc(TILEMAP_DATA_SIZE*sizeof(int));
  if(!map->data)
  {
      free (map);  // added by Amarillion
      return NULL;
  }
  fread(map->data, sizeof(int), TILEMAP_DATA_SIZE, mapfile);

  if(map->names)
  {
    map->name = malloc(NAMELIST_SIZE);
    for(count=0; count<map->names; count++)
    {
      fread(&map->name[count].index, sizeof(int), 1, mapfile);
      fgets(map->name[count].name, MAX_STRING_LENGTH, mapfile);
      map->name[count].name[strlen(map->name[count].name)-1] = 0;
    }
  }

  fclose(mapfile);

  return map;
}


// --------------------------------------------------------------------------
// The register_tilemap function

TILEMAP *register_tilemap(DATAFILE *dat, int index)
{
  TILEMAP *map = dat[index].dat;
  char    *list_ptr;
  int     count;

  // initialize tilemap for use with allegro
  map->data = (int*)  ((char*) map+sizeof(TILEMAP));
  list_ptr  = (char*) ((char*) map+sizeof(TILEMAP)+TILEMAP_DATA_SIZE*sizeof(int));

  if(map->names)
  {
    map->name = malloc(NAMELIST_SIZE);
    for(count=0; count<map->names; count++)
    {
      map->name[count].index = *list_ptr;
      list_ptr += sizeof(int);
      list_ptr[strcspn(list_ptr, "\n")] = 0;
      strcpy(map->name[count].name, list_ptr);
      list_ptr += strlen(list_ptr)+1;
    }
  }

  return map;
}


// --------------------------------------------------------------------------
// The save_tilemap function

void save_tilemap(TILEMAP *map, char *filename)
{
  FILE *mapfile;
  int  count;

  mapfile = fopen(filename, "wb");

  fwrite(map, sizeof(TILEMAP), 1, mapfile);
  fwrite(map->data, sizeof(int), TILEMAP_DATA_SIZE, mapfile);

  for(count=0; count<map->names; count++)
  {
    fwrite(&(map->name[count].index), sizeof(int), 1, mapfile);
    fputs(map->name[count].name, mapfile);
    fputc('\n', mapfile);
  }
  
  fclose(mapfile);
}


// --------------------------------------------------------------------------
// The create_tilemap function

TILEMAP *create_tilemap(int layers, int xsize, int ysize, int xgrid, int ygrid)
{
  TILEMAP *map;
  int     counter;

  // allocate stuff
  map          = malloc(sizeof(TILEMAP));
  map->version = TILEMAP_VERSION;
  map->xgrid   = xgrid;
  map->ygrid   = ygrid;
  map->xsize   = xsize;
  map->ysize   = ysize;
  map->layers  = layers;
  map->names   = 0;
  map->data    = malloc(TILEMAP_DATA_SIZE*sizeof(int));
  map->name    = NULL;

  // clear tilemap
  for(counter=0; counter<TILEMAP_DATA_SIZE; counter++)
  map->data[counter] = NO_SPRITE;

  return map;
}


// --------------------------------------------------------------------------
// The resize_tilemap function

TILEMAP *resize_tilemap(TILEMAP *map, int layers, int xsize, int ysize, int xgrid, int ygrid, int xcut, int ycut)
{
  TILEMAP *tempmap;
  int     lcount, xcount, ycount;

  if(map)
  {
    // make a new resized tilemap
    tempmap = create_tilemap(layers, xsize, ysize, xgrid, ygrid);

    for(lcount=0; lcount<tempmap->layers; lcount++)
    for(xcount=0; xcount<tempmap->xsize ; xcount++)
    for(ycount=0; ycount<tempmap->ysize ; ycount++)
    {
      tempmap->data[get_sprite_pos(tempmap, lcount, xcount, ycount)] =
      get_sprite_number(map, lcount,
      (xcut) ? (xcount+map->xsize-xsize) : xcount,
      (ycut) ? (ycount+map->ysize-ysize) : ycount);
    }
    
    destroy_tilemap(map);
    return tempmap;
  }
  else return NULL;
}


// --------------------------------------------------------------------------
// The destroy_tilemap function

void destroy_tilemap(TILEMAP *map)
{
  if(map)
  {
    if(map->data) free(map->data);
    if(map->name) free(map->name);
    free(map);
  }
}


// --------------------------------------------------------------------------
// The copy_layer function

int copy_tilemap_layer(TILEMAP *map, int src_layer, int dest_layer)
{
  if((src_layer <= map->layers) && (dest_layer <= map->layers) && (src_layer != dest_layer))
    memcpy(&map->data[(TILEMAP_LAYER_SIZE)*dest_layer], &map->data[(TILEMAP_LAYER_SIZE)*src_layer], TILEMAP_LAYER_SIZE*sizeof(int));
  else
    return FALSE;

  return TRUE;
}


// --------------------------------------------------------------------------
// The draw_tilemap_layer function

void draw_tilemap_layer(BITMAP *dest_bmp, DATAFILE *dat, TILEMAP *map, int layer, int dest_x, int dest_y)
{
  int sprite, x, y;
  
  for(x = dest_x>0 ? 0 : -dest_x/map->xgrid; (x*map->xgrid+dest_x<dest_bmp->w) && (x<map->xsize); x++) {
    for(y = dest_y>0 ? 0 : -dest_y/map->ygrid; (y*map->ygrid+dest_y<dest_bmp->h) && (y<map->ysize); y++) {
      sprite = get_sprite_number(map, layer, x, y);
      if(sprite != NO_SPRITE) {
        if(sprite != SOLID_TILE) draw_rle_sprite(dest_bmp, (RLE_SPRITE*) dat[sprite].dat, dest_x+x*map->xgrid, dest_y+y*map->ygrid); // modified by MI
	else {
	  drawing_mode(DRAW_MODE_TRANS, NULL, 0, 0);
	  rectfill(dest_bmp, dest_x+x*map->xgrid, dest_y+y*map->ygrid, dest_x+(x+1)*map->xgrid-1, dest_y+(y+1)*map->ygrid-1, makecol(0, 0, 0));
	  solid_mode();
	}
      }
    }
  }
}


// --------------------------------------------------------------------------
// The get_sprite_number function

int get_sprite_number(TILEMAP *map, int layer, int x, int y)
{
  if((layer < map->layers) && (x < map->xsize) && (y < map->ysize))
    return map->data[get_sprite_pos(map, layer, x, y)];
  else
    return NO_SPRITE;
}


// --------------------------------------------------------------------------
// The set_sprite_number function

void set_sprite_number(TILEMAP *map, int layer, int x, int y, int sprite_num)
{
  if((layer < map->layers) && (x < map->xsize) && (y < map->ysize))
  map->data[get_sprite_pos(map, layer, x, y)] = sprite_num;
}


// --------------------------------------------------------------------------
// The get_sprite_pos function

int get_sprite_pos(TILEMAP *map, int layer, int x, int y)
{
  return (x+y*map->xsize) + (map->xsize*map->ysize*layer);
}


// --------------------------------------------------------------------------
// The bb_collission function

int bb_collission(int x1, int y1, int w1, int h1, int x2, int y2, int w2, int h2)
{
  if((x1 >= x2+w2) || (x2 >= x1+w1) || (y1 >= y2+h2) || (y2 >= y1+h1)) return FALSE;

  return TRUE;
}


// --------------------------------------------------------------------------
// The st_collission function

int st_collission(TILEMAP *map, int layer, int x, int y, int w, int h)
{
  int xc, yc;
  int sprite;

  // see if there are any solid tiles under the object
  for(xc = x/map->xgrid-1; xc <= (x+w)/map->xgrid+1; xc++) {
    for(yc = y/map->ygrid-1; yc <= (y+h)/map->ygrid+1; yc++) {
      sprite = get_sprite_number(map, layer, xc, yc);
      if(sprite == SOLID_TILE)
      if(bb_collission(xc*map->xgrid, yc*map->ygrid, map->xgrid, map->ygrid, x, y, w, h)) return TRUE;
    }
  }

  return FALSE;
}


// --------------------------------------------------------------------------
// The create_namelist function

void create_namelist(TILEMAP *map, DATAFILE *dat)
{
  int count, names;

  // count up how many sprites the datafile contains
  names = 0;
  for(count = 0; dat[count].type != DAT_END; count++)
  if(dat[count].type == DAT_RLE_SPRITE) names++;

  // create namelist
  map->names = names;
  map->name  = malloc(NAMELIST_SIZE);

  // fill namelist
  names = 0;
  for(count = 0; dat[count].type != DAT_END; count++)
  if(dat[count].type == DAT_RLE_SPRITE)
  {
    map->name[names].index = count;
    strcpy(map->name[names].name, get_object_name(dat, count));
    names++;
  }
}


// --------------------------------------------------------------------------
// The correct_tilemap function

void correct_tilemap(TILEMAP *map, DATAFILE *dat)
{
  INDEXLIST *indexlist = malloc(sizeof(INDEXLIST)*map->names);
  int       count, count2;
  
  // build indexlist
  for(count=0; count<map->names; count++)
  {
    indexlist[count].from = map->name[count].index;
    indexlist[count].to   = NO_SPRITE;

    for(count2=0; dat[count2].type != DAT_END; count2++)
    {
      if(!strcmp(map->name[count].name, get_object_name(dat, count2)))
      {
        indexlist[count].to = count2;
        break;
      }
    }
  }

  // correct tilemap using the indexlist
  for(count=0; count<TILEMAP_DATA_SIZE; count++)
  {
    for(count2=0; count2<map->names; count2++)
    {
      if(map->data[count] == indexlist[count2].from)
      {
        map->data[count] = indexlist[count2].to;
        break;
      }
    }
  }

  // clean up
  free(indexlist);
}

// --------------------------------------------------------------------------
// End of file


