//     :     :     :     :     :     :     :     :     :     :
//   --------:  ---------:  ---------:  ---------:  ----     :
// ..|      |*..|       |*..|       |*..|       |*..|  |*....:.
//   |      |*  |       |*  |       |*  |       |*  |  |*    :
//   ---  ---*  |  |-----*  |  |-----*  |  |-----*  |  |*    :
// ....|  |*.:..|  |--****..|  |-----*..|  |--****..|  |*....:.
//     |  |* :  |    |*  :  |  |--  |*  |    |*  :  |  |*    :
//     |  |* :  |  |--*  :  |  |*|  |*  |  |--*  :  |  |*    :
// ....|  |*.:..|  |-----:..|  ---  |*..|  |-----:..|  -----.:.
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
//     |  |* :  |       |*  |       |*  |       |*  |      |*:
// ....----*.:..---------*..---------*..---------*..--------*:.
//     :**** :   *********   *********   *********   ********:
//     :     :     :     :     :     :     :     :     :     :
//
// Version 0.81
// Copyright (c) 2002-2007
// Martijn "Amarillion" van Iersel.
//
// This file is part of the Tegel tilemapping add-on for allegro
//
//    Tegel is free software; you can redistribute it and/or modify
//    it under the terms of the GNU Lesser General 
//    Public License as published by the Free Software 
//    Foundation; either version 2.1 of the License, or
//    (at your option) any later version.
//
//    Tegel is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public License
//    along with Tegel; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
// official website:
// http://tegel.sourceforge.net/
//
// I hope you like this program. Please send 
// questions, comments, flames and feature 
// requests to amarillion@yahoo.com.
//
// --------------------------------------------------------------------------
// Level-Editor v1.08 --- Tilemap Library header file.
// Copyright (c) Henrik Stokseth, 2000.
// modifications by Amarillion

#ifndef __TILEMAP__
#define __TILEMAP__

// added by Amarillion
#ifdef __cplusplus
    extern "C" {
#endif

// --------------------------------------------------------------------------
// Include files

#include <allegro.h>


// --------------------------------------------------------------------------
// Definitions

//added by Amarillion
#define DAT_MAPDATA  DAT_ID('M','A','P', ' ')

#define TILEMAP_VERSION    1
#define TILEMAP_DATA_SIZE  (map->xsize * map->ysize * map->layers)
#define TILEMAP_LAYER_SIZE (map->xsize * map->ysize)
#define NAMELIST_SIZE      (sizeof(NAMELIST)*map->names)

#define NO_SPRITE          -1
#define SOLID_TILE         -2

#define MAX_STRING_LENGTH  80


// --------------------------------------------------------------------------
// Structure definitions

typedef struct NAMELIST
{
  int  index;                   // the sprite's datafile index number
  char name[MAX_STRING_LENGTH]; // the sprite's datafile name
} NAMELIST;

typedef struct INDEXLIST
{
  int from; // old datafile index number
  int to;   // new datafile index number
} INDEXLIST;

typedef struct TILEMAP
{
  int      version;      // map-file version
  int      xsize, ysize; // size of map measured in tiles
  int      xgrid, ygrid; // size of grid measured in pixels
  int      layers;       // number of layers
  int      names;        // number of sprite-names (used for auto-correction)
  int      *data;        // map data
  NAMELIST *name;        // sprite names (used for auto-correction)
} TILEMAP;


// --------------------------------------------------------------------------
// Function prototypes

// functions added by Amarillion
int find_sprite_in_box (TILEMAP *map, int layer, int x, int y, int w, int h, char prop[]);
int tilemap_data_size (TILEMAP* map);
int tilemap_namelist_size (TILEMAP *map);
void *load_mapdata(PACKFILE *f, long size);
void destroy_mapdata(void *data);
void register_datafile_tilemap ();

// datafile helper functions  (mostly for internal use)
char *get_object_name(DATAFILE *dat, int objnum);
int  object_of_type_exists(DATAFILE *dat, int type);
int  number_of_type(DATAFILE *dat, int type);
int  get_object_number(DATAFILE *dat, int number_of_type, int type);
int  get_max_sprite_width(DATAFILE *dat);
int  get_max_sprite_height(DATAFILE *dat);
TILEMAP *copy_tilemap (TILEMAP* original);

// tilemap helper functions
TILEMAP *load_tilemap(char *filename);
//note: register_tilemap has become obsolete (Amarillion)
TILEMAP *register_tilemap(DATAFILE *dat, int index);
void    save_tilemap(TILEMAP *map, char *filename);
TILEMAP *create_tilemap(int layers, int xsize, int ysize, int xgrid, int ygrid);
TILEMAP *resize_tilemap(TILEMAP *map, int layers, int xsize, int ysize, int xgrid, int ygrid, int xcut, int ycut);
void    destroy_tilemap(TILEMAP *map);
int     copy_tilemap_layer(TILEMAP *map, int src_layer, int dest_layer);


// drawing helper functions
void draw_tilemap_layer(BITMAP *dest_bmp, DATAFILE *dat, TILEMAP *map, int layer, int dest_x, int dest_y);

// tile helper functions
int  get_sprite_number(TILEMAP *map, int layer, int x, int y);
void set_sprite_number(TILEMAP *map, int layer, int x, int y, int sprite_num);
int  get_sprite_pos(TILEMAP *map, int layer, int x, int y);

// collission detection helper functions

int bb_collission(int x1, int y1, int w1, int h1, int x2, int y2, int w2, int h2);
int st_collission(TILEMAP *map, int layer, int x, int y, int w, int h);

// auto-correction helper functions
void create_namelist(TILEMAP *map, DATAFILE *dat);
void correct_tilemap(TILEMAP *map, DATAFILE *dat);

//added by Amarillion
#ifdef __cplusplus
   }
#endif

#endif // __TILEMAP__

// --------------------------------------------------------------------------
// End of header file
